var MWearGaugeTreeActions = AMTreeActions.extend({
   initialize: function() {
       var oOverallStatus = {
         'nCritical': 0,
         'nWarn': 0,
         'nInfo': 0         
      };
      var oSSDDevices = [];
      var oDevices = [];
      var oControllers = [];
      this.set({
         'sEmptyMsg':'STR_EMPTY_ARRAYS',
         "sTitle": "STR_SSD_DEVICES",
         'oOverallStatus': oOverallStatus,
         'oSSDDevices': oSSDDevices,
         'oDevices': oDevices,
         'oControllers': oControllers
      })
      this.findSSDDevices();
      AMTab1.prototype.initialize.call(this);
   },

   findSSDDevices: function() {
      var oParents = this.get('oReport').DeviceList;
      for (var i = 0; i < oParents.length; ++i)
      {
         this.get('oControllers')[i] = oParents[i];
         //printJaveObject(m_oCtrls[i]);
         this.findSSDs(oParents[i]);
      }
   },

   findSSDs: function(oDevice, oParent) {
      var oSmartSSD = oDevice.SmartSSDWearGauge;
      if (IsDefined(oSmartSSD))
      {
         var oSSDDevices = this.get('oSSDDevices');
         oSSDDevices[oSSDDevices.length] = this.createSSDObject(oParent, oDevice);
         var oDevices = this.get('oDevices');
         oDevices[oDevices.length] = oDevice.id;
      }
      else
      {
         var oDL = oDevice.DeviceList;
         if (IsDefined(oDL))
         {
            var bIncludeParent = oDevice.deviceType == "ArrayController" ||
                                 oDevice.deviceType == "NonSmartArrayController" ||
                                 oDevice.deviceType == "HBA";
            for (var i = 0; i < oDL.length; ++i)
            {
               if (bIncludeParent)
               {
                  this.findSSDs(oDL[i], oDevice);
               }
               else
               {
                  this.findSSDs(oDL[i], oParent);
               }
            }
         }
      }
   },

   createSSDObject: function(oController, oDevice) {
      var oSSD = [];
      oSSD.oController = oController;
      oSSD.oDevice = oDevice;
      oSSD.sStatusText = "";
      oSSD.nSSDStatus = 999; // 1=Failed 2=Warning 3=Informational 4=OK
      oSSD.sPhysicalStatus;
      oSSD.bHasAnyStatus = false;
      oSSD.sDaysRemaining = "";
      oSSD.sUsageRemaining = "-1";
      oSSD.nSize = "";
      oSSD.sInterface = "";
      oSSD.sInterfaceShort = "";
      oSSD.sAttrBox = "";
      oSSD.sAttrBay = "";
      oSSD.sAttrPort = "";
      oSSD.sLocation = "";
      oSSD.sLabel = "";
      oSSD.sCtrlType = "";
      oSSD.bSmartSSDSupported;

      var oOverallStatus = this.get('oOverallStatus');

      // Days remaining
      oSSD.sDaysRemaining = this.getValue(oDevice.SmartSSDWearGauge, "Remaining Days Until Wearout");

      // Usage Remaining
      var nUtilization = this.getValue(oDevice.SmartSSDWearGauge, "Utilization");
      var bValidLifeLeft = IsDefined(nUtilization) && nUtilization >= 0 && nUtilization <= 100;
      if (bValidLifeLeft)
      {
         oSSD.sUsageRemaining = 100 - nUtilization;
      }

      // Physical Status
      oSSD.sPhysicalStatus = this.getValue(oDevice.SmartSSDWearGauge, "Status");
      if (IsDefined(oSSD.sPhysicalStatus)) {
         oSSD.sPhysicalStatus = TranslateString(oSSD.sPhysicalStatus);
      }

      // SmartSSDSupported
      var sSupported = this.getValue(oDevice.SmartSSDWearGauge, "Supported");
      if (!IsDefined(sSupported) || sSupported == "FALSE")
      {
         oSSD.bSmartSSDSupported = false;
      }
      else
      {
         oSSD.bSmartSSDSupported = true;
      }

      // Status
      var sStatus = this.getValue(oDevice.SmartSSDWearGauge, "Status");
      var sSmartTrip = this.getValue(oDevice.SmartSSDWearGauge, "Has Smart Trip SSD Wearout");
      var sUtilizationWarning = this.getValue(oDevice.SmartSSDWearGauge, "Has Utilization Warning");
      var s56DaysWarning = this.getValue(oDevice.SmartSSDWearGauge, "Has 56 Day Warning");
      var bSmartTrip = IsDefined(sSmartTrip) && sSmartTrip == "TRUE";
      if (IsDefined(sStatus) && sStatus == "FAILED")
      {
         oSSD.nSSDStatus = 1; // This is Failed level
         oSSD.bHasAnyStatus = true;
         oSSD.sStatusText = sStatus;
         if (bSmartTrip)
         {
            oSSD.sStatusText = "STR_MSG_SSD_SMART_TRIP";
         }
         oOverallStatus.nCritical++;
      }
      else if (bSmartTrip)
      {
         oSSD.nSSDStatus = 2; // This is warning level
         oSSD.bHasAnyStatus = true;
         oSSD.sStatusText = "STR_MSG_SSD_SMART_TRIP";

         oOverallStatus.nWarn++;
      }
      else
      {
         if (IsDefined(s56DaysWarning) && s56DaysWarning == "TRUE")
         {
            oSSD.nSSDStatus = 3; // This is info level
            oSSD.bHasAnyStatus = true;
            oSSD.sStatusText = "STR_MSG_SSD_56DAY_WARNING";
         }
         if (IsDefined(sUtilizationWarning) && sUtilizationWarning != "NONE")
         {
            oSSD.nSSDStatus = 3; // This is info level
            oSSD.bHasAnyStatus = true;
            if (sUtilizationWarning == "5PERCENT")
            {
               if (oSSD.nSSDStatus != 999)
               {
                  oSSD.sStatusText = "STR_MSG_SSD_56_DAY_5PERCENT_LIFE_REMAINING";
               }
               else
               {
                  oSSD.sStatusText = "STR_MSG_SSD_5PERCENT_LIFE_REMAINING";
               }
            }
            else if (sUtilizationWarning == "2PERCENT")
            {
               if (oSSD.nSSDStatus != 999)
               {
                  oSSD.sStatusText = "STR_MSG_SSD_56_DAY_2PERCENT_LIFE_REMAINING";
               }
               else
               {
                  oSSD.sStatusText = "STR_MSG_SSD_2PERCENT_LIFE_REMAINING";
               }
            }
         }

         if (oSSD.nSSDStatus == 3)
         {
            oOverallStatus.nInfo++;
         }
      }

      // Check for other warning status such as PREDICTIVE_FAILURE, rebuilding...
      if (IsDefined(sStatus) && sStatus != "OK")
      {
         if (oSSD.nSSDStatus == 999)
         {
            oOverallStatus.nWarn++;
         }
         if (oSSD.nSSDStatus == 3)
         {
            oOverallStatus.nInfo--;
            oOverallStatus.nWarn++;
         }
         oSSD.bHasAnyStatus = true;
         //oSSD.nSSDStatus = 2; // This is warning level
      }

      // Check for log full
      var sSmartSSDLogFull = this.getValue(oDevice.SmartSSDWearGauge, "Log Full");
      if (IsDefined(sSmartSSDLogFull) && sSmartSSDLogFull == "TRUE")
      {
         if (oSSD.nSSDStatus == 999)
         {
            oOverallStatus.nWarn++;
         }
         if (oSSD.nSSDStatus == 3)
         {
            oOverallStatus.nInfo--;
            oOverallStatus.nWarn++;
         }
         oSSD.bHasAnyStatus = true;
         //oSSD.nSSDStatus = 2; // This is warning level
      }

      // If still don't have a status, it must be ok.
      if (oSSD.nSSDStatus == 999)
      {
         oSSD.nSSDStatus = 4; // This is OK level
         oSSD.sStatusText = "OK";
      }
      if (oSSD.sStatusText == "")
      {
         oSSD.sStatusText = "OK";
      }

      var sSSDType = oDevice.deviceType;
      var oPhysicalDriv, sPhysicalDrive;
      var sSSDInterface, sSerialNumber, sFirmwareRevision;
      if (IsDefined(sSSDType))
      {
         if (sSSDType == "NonSmartArrayPhysicalDrive")
         {
            oSSD.sCtrlType = "NON_SMART";
            oPhysicalDrive = oDevice.PhysicalDriveInfo;
            sPhysicalDrive = "PhysicalDriveInfo";
            var interfaceValue = this.getValue(oPhysicalDrive, "Interface");
            if (interfaceValue == "SAS_SSD")
            {
               oSSD.sInterface = TranslateString("STR_SAS_SSD_LONG");
               oSSD.sInterfaceShort = TranslateString("STR_SAS_SSD_SHORT");
            }
            else
            {
               oSSD.sInterface = TranslateString("STR_SATA_SSD_LONG");
               oSSD.sInterfaceShort = TranslateString("SATA STR_SATA_SSD_SHORT");
            }
         }
         else
         {
            oSSD.sCtrlType = "SMART";
            oPhysicalDrive = oDevice.PhysicalDriveStatus;
            sPhysicalDrive = "PhysicalDriveStatus";
            var interfaceValue = this.getValue(oPhysicalDrive, "Device Type");
            if (interfaceValue == "0x02")
            {
               oSSD.sInterface = TranslateString("STR_SAS_SSD_LONG");
               oSSD.sInterfaceShort = TranslateString("STR_SAS_SSD_SHORT");
            }
            else
            {
               oSSD.sInterface = TranslateString("STR_SATA_SSD_LONG");
               oSSD.sInterfaceShort = TranslateString("STR_SATA_SSD_SHORT");
            }
         }
      }

      // Size
      var size = parseInt(this.getValue(oPhysicalDrive, "Total Blocks"));
      oSSD.nSize = GUIConvertSize(size);

      // Location
      var sAttrBox = this.getValue(oPhysicalDrive, "Physical Box");
      var sAttrBay = this.getValue(oPhysicalDrive, "Physical Bay");
      var sAttrPort = this.getValue(oPhysicalDrive, "Physical Port");
      if (IsDefined(sAttrBox))
      {
         // Convert to int
         oSSD.sAttrBox = parseInt(sAttrBox);
         oSSD.sAttrBay = parseInt(sAttrBay);
         oSSD.sAttrPort = sAttrPort;
         oSSD.sLocation = TranslateString("STR_PORT_BOX_BAY|" + oSSD.sAttrPort + "|" + oSSD.sAttrBox + "|" + oSSD.sAttrBay);
      }
      return oSSD;
   },

   getValue: function(o, sAttrName) {
      var sValue;
      for (i=0; i<o.length; ++i)
      {
         if (o[i].id == sAttrName)
         {
            sValue = o[i].value;
         }
      }
      return sValue;
   }

});