var VWearGaugeTab = AVTreeTab.extend({

   events: {
      "click #viewStatusMessages":"drawDetailedStatus"
   },

   initialize: function() {
      AVTreeTab.prototype.initialize.call(this);
      var oSSDDevices = this.model.get('oSSDDevices');
      var oSSDStatusAlerts = [];
      for (var i=0; i<oSSDDevices.length; ++i) {
         if (oSSDDevices[i].bHasAnyStatus) {
            var sDeviceID = oSSDDevices[i].oDevice.id;
            var sControllerID = oSSDDevices[i].oController.id;
            var oDeviceDetails = GetDeviceDetails(sDeviceID, 1).device;
            for (var j=0; j<oDeviceDetails.length; ++j) {
               if (oDeviceDetails[j].SectionName == "STATUS") {
                  oSSDStatusAlerts[oSSDStatusAlerts.length] = {
                     'id':sDeviceID,
                     'cid':sControllerID,
                     'alerts':oDeviceDetails[j]
                  };
               }
            }
         }
      }
      this.model.set({
         'oSSDStatusAlerts':oSSDStatusAlerts
      });
   },

   drawAll: function() {
      var sDeviceID = this.model.get('sDeviceID');
      var sDeviceType = "";
      if (sDeviceID == '') {
         sDeviceType = "overview";
      }
      else if (sDeviceID == 'status') {
         sDeviceType = "status";
      }
      else {
         sDeviceType = GetDeviceType(this.model.get('sDeviceID'));
      }
      var oBodyDiv = this.$('.opBodyInner');
      this.$('.leftColumn').remove();
      var oColumnDiv = $('<div/>', {'class':'leftColumn'});
      oBodyDiv.append(oColumnDiv);
      if (sDeviceType == "overview") {
         // Display overview page for Arrays.
         this.drawOverviewHeader();
         this.drawStatusWidget(oColumnDiv);
         this.drawSummaryWidget(oColumnDiv);
         this.$('.opFooter').remove();
         TriggerResize();
      }
      else if (sDeviceType == "status") {
         this.drawStatusHeader();
         //this.drawStatusWidget(oColumnDiv);
         this.drawAllStatusesWidget(oColumnDiv);
         this.drawFooter();
         this.drawButtons();
         TriggerResize();
      }
      else {
         this.drawHeader();
         this.drawDetailsWidget(oColumnDiv);
         this.drawFooter();
         this.drawButtons();
      }
   },

   drawOverviewHeader: function() {
      var oHeaderDiv = this.$('.opHeader').empty();
      var oDeviceLabelDiv = $('<div/>', {'class': 'deviceLabel'});
      oHeaderDiv.append(oDeviceLabelDiv);
      var oMDeviceLabel = new MDeviceLabel({
         'sDeviceID': '',
         'sLabel': TranslateString("STR_WEAR_GAUGE_SUMMARY"),
         'sIconClass': "ICON_SSD_WEAR_GAUGE"
      });
      var oVDeviceLabel = new VDeviceLabel({
         'model': oMDeviceLabel,
         'el': oDeviceLabelDiv
      });
      oHeaderDiv.append(GetClearFloatsDiv());
      //oParentDiv.append(oHeaderDiv);
   },

   drawStatusHeader: function() {
      var oHeaderDiv = this.$('.opHeader').empty();
      var oDeviceLabelDiv = $('<div/>', {'class': 'deviceLabel'});
      oHeaderDiv.append(oDeviceLabelDiv);
      var oMDeviceLabel = new MDeviceLabel({
         'sDeviceID': '',
         'sLabel': TranslateString("STR_SMARTSSD_STATUS_ALERTS"),
         'sIconClass': "ICON_SSD_WEAR_GAUGE"
      });
      var oVDeviceLabel = new VDeviceLabel({
         'model': oMDeviceLabel,
         'el': oDeviceLabelDiv
      });
      oHeaderDiv.append(GetClearFloatsDiv());
      //oParentDiv.append(oHeaderDiv);
   },

   drawButtons: function() {
      var oButtonsDiv = this.$('.buttons');
      oButtonsDiv.empty();
      var sLabel = 'STR_BACK_TO_OVERVIEW';
      var sClass = "button popupTabbable pianoButton default BackToOverview";
      var oButtonDiv = $('<a/>', {
         'href':'',
         'id':sLabel,
         'class':sClass
      });
      oButtonDiv.html(TranslateString(sLabel));
      oButtonsDiv.append(oButtonDiv);
   },

   drawStatusWidget: function(oParentDiv) {
      var oWidgetDiv = this.drawColumnWidget(oParentDiv, "statusWidget");
      var oBox = this.drawBox(oWidgetDiv);
      var oTitle = $('<div/>', {'class': 'title'});
      oTitle.html(TranslateString("STR_OVERALL_STATUS"));
      oBox.append(oTitle);
      oBox.append(GetClearFloatsDiv());
      var oStatusDiv = $('<div/>', {'class': 'statusDiv'});
      this.drawCounts(oStatusDiv);
      oBox.append(oStatusDiv);
   },

   drawAllStatusesWidget: function(oParentDiv) {
      var oWidgetDiv = this.drawColumnWidget(oParentDiv, "statusWidget");
      var oBox = this.drawBox(oWidgetDiv);
      var oTitle = $('<div/>', {'class': 'title'});
      oTitle.html(TranslateString("STR_OVERALL_STATUS"));
      oBox.append(oTitle);
      oBox.append(GetClearFloatsDiv());
      var oStatusDiv = $('<div/>', {'class': 'statusDiv'});
      this.drawCounts(oStatusDiv, false);
      oBox.append(oStatusDiv);
      var oStatusDetailsDiv = $('<div/>', {'class': 'statusDetailsDiv'});
      this.drawStatusDetails(oStatusDetailsDiv);
      oBox.append(oStatusDetailsDiv);
   },

   drawCounts: function(oParentDiv, bDrawLink) {
      if (!IsDefined(bDrawLink)) {
         bDrawLink = true;
      }
      var oOverallStatus = this.model.get('oOverallStatus');
      var sCriticalClass = oOverallStatus.nCritical > 0 ? "criticalActive" : "criticalInactive";
      var sWarnClass = oOverallStatus.nWarn > 0 ? "warnActive" : "warnInactive";
      var sInfoClass = oOverallStatus.nInfo > 0 ? "infoActive" : "infoInactive";
      
      // Add Status Alert Counts Table
      var oSectionAlertsTable = $('<table/>', {'class': 'sectionAlerts'});
      var oTR = $('<tr/>');
      var oTD = $('<td/>', {'class': 'statusCounts'});
      var s = 
         '<div class="statusIcon ' + sCriticalClass + ' HPSSA_SPRITE"></div><div class="value">' + oOverallStatus.nCritical + '</div>' +
         '<div class="statusIcon ' + sWarnClass + ' HPSSA_SPRITE"></div><div class="value">' + oOverallStatus.nWarn + '</div>' +
         '<div class="statusIcon ' + sInfoClass + ' HPSSA_SPRITE"></div><div class="value">' + oOverallStatus.nInfo + '</div>';
      oTD.html(s);
      oTR.append(oTD);
      oSectionAlertsTable.append(oTR);
      oParentDiv.append(oSectionAlertsTable);
      if (bDrawLink) {
         var oLink = $('<a/>', {
            'href':'',
            'id':'viewStatusMessages', 
            'class':'viewAllStatuses'
         });
         oLink.html(TranslateString("STR_VIEW_ALL_STATUS_MESSAGES"));
         oParentDiv.append(oLink);
      }
   },

   drawStatusDetails: function(oParentDiv) {
      // Get alert counts
      var oOverallStatus = this.model.get('oOverallStatus');
      var nCritical = oOverallStatus.nCritical;
      var nWarn = oOverallStatus.nWarn;
      var nInfo = oOverallStatus.nInfo;
      // Get all the status alerts
      var oSSDStatusAlerts = this.model.get('oSSDStatusAlerts');
      var oMsgs = $('<div/>', {'class': 'sectionMessages'});
      // Create arrays for each type of alert
      var fatalCodes = [];
      var fatalMessages = [];
      var failCodes = [];
      var failMessages = [];
      var warnCodes = [];
      var warnMessages = [];
      var infoCodes = [];
      var infoMessages = [];
      for (var i=0; i<oSSDStatusAlerts.length; ++i) {
         var sControllerID = oSSDStatusAlerts[i].cid;
         var sControllerName = TranslateString(GetDeviceIDSection(sControllerID).NAME);
         var oStatus = oSSDStatusAlerts[i].alerts;
         if (oStatus.SectionName == "STATUS") {
            if (oStatus.STATUS_CODES_FATAL != "") {
               var oStatusCodes = this.prepStatusCodes(oStatus.STATUS_CODES_FATAL, sControllerName);
               $.merge(fatalCodes, oStatusCodes);
               $.merge(fatalMessages, oStatus.STATUS_MESSAGES_FATAL.split("||"));
            }
            if (oStatus.STATUS_CODES_FAIL != "") {
               var oStatusCodes = this.prepStatusCodes(oStatus.STATUS_CODES_FAIL, sControllerName);
               $.merge(failCodes, oStatusCodes);
               $.merge(failMessages, oStatus.STATUS_MESSAGES_FAIL.split("||"));
            }
            if (oStatus.STATUS_CODES_WARN != "") {
               var oStatusCodes = this.prepStatusCodes(oStatus.STATUS_CODES_WARN, sControllerName);
               $.merge(warnCodes, oStatusCodes);
               $.merge(warnMessages, oStatus.STATUS_MESSAGES_WARN.split("||"));
            }
            if (oStatus.STATUS_CODES_INFO != "") {
               var oStatusCodes = this.prepStatusCodes(oStatus.STATUS_CODES_INFO, sControllerName);
               $.merge(infoCodes, oStatusCodes);
               $.merge(infoMessages, oStatus.STATUS_MESSAGES_INFO.split("||"));
            }
         }
      }
      // Add Fatal Messages.
      //this.drawStatusSection(oMsgs, "Fatal Error(s)", "ICON_OVERLAY_STATUS_FATAL", nFatal, fatalCodes, fatalMessages);
      // Add Fail/Critical Messages.
      this.drawStatusSection(oMsgs, "STR_MSG_STATUS_CRITICAL", "ICON_OVERLAY_STATUS_FATAL", nCritical, failCodes, failMessages);
      // Add Warn Messages.
      this.drawStatusSection(oMsgs, "STR_MSG_STATUS_WARNING", "ICON_OVERLAY_STATUS_WARN", nWarn, warnCodes, warnMessages);
      // Add Info Messages.
      this.drawStatusSection(oMsgs, "STR_MSG_STATUS_INFO", "ICON_OVERLAY_STATUS_INFO", nInfo, infoCodes, infoMessages);
      oParentDiv.append(oMsgs);
   },

   prepStatusCodes: function(oRawStatusCodes, sControllerName) {
      var oStatusCodes = oRawStatusCodes.split("|");
      for (var j=0; j<oStatusCodes.length; ++j) {
         oStatusCodes[j] = oStatusCodes[j] + "|" + sControllerName;
      }
      return oStatusCodes;
   },

   drawStatusSection: function(oParentDiv, sTitle, sIcon, countVar, codesVar, messagesVar) {
      if ( countVar > 0 )
      {
         var oSectionDiv = $('<div/>', {'class':'section'});
         var oSectionTable = $('<table/>', {'class':'section'});
         var oTitleDiv = $('<div/>', {'class':'sectionTitle'});
         var sIconClass = 'icon HPSSA_SPRITE ' + sIcon;
         var oIconDiv = $('<div/>', {'class':sIconClass});
         var oNameDiv = $('<div/>', {'class':'name'});
         oNameDiv.append(TranslateString(sTitle));
         oTitleDiv.append(oIconDiv);
         oTitleDiv.append(oNameDiv);
         oSectionDiv.append(oTitleDiv);
         for ( var i = 0; i < countVar; ++i )
         {
            var sTRClass = 'row';
            if (i == countVar - 1)
            {
               sTRClass += ' last';
            }
            var oRowTR = $('<tr/>', {'class':sTRClass});
            
            // Add Icon
            /*var sIconClass = 'icon HPSSA_SPRITE SMALL ' + sIcon;
            var oIconTD = $('<td/>', {'class':'icon', 'valign':'top'});
            var oIconDiv = $('<div/>', {'class':sIconClass});
            oIconTD.append(oIconDiv);
            oRowTR.append(oIconTD);*/

            var oCodeTD = $('<td/>', {'class':'code', 'valign':'top'});
            var oCodes = codesVar[i].split("|");
            oCodeTD.append(oCodes[0]);
            oRowTR.append(oCodeTD);

            // Add Message
            var oMsgTD = $('<td/>', {'class':'msg', 'valign':'top'});
            var sMessage = oCodes[1] + ": " + TranslateStatusMessage(messagesVar[i]);
            oMsgTD.html(sMessage);
            oRowTR.append(oMsgTD);
            oSectionTable.append(oRowTR);
         }
         oSectionDiv.append(oSectionTable);
         oParentDiv.append(oSectionDiv);
         oTitleDiv.click(function() {
            oSectionTable.toggle();
         });
      }
   },

   drawSummaryWidget: function(oParentDiv) {
      var oWidgetDiv = this.drawColumnWidget(oParentDiv, "detailsWidget");
      var oBox = this.drawBox(oWidgetDiv);

      var oSSDWearoutList = this.getSSDs("Has Smart Trip SSD Wearout", "TRUE");
      var oSSD56DaysList = this.getSSDs("Has 56 Day Warning", "TRUE");
      var oSSD5PercentList = this.getSSDs("Has Utilization Warning", "5PERCENT");
      var oSSD2PercentList = this.getSSDs("Has Utilization Warning", "2PERCENT");
      var oSSDs = this.model.get("oSSDDevices");
      var nTotalWearouts = oSSDWearoutList.length;
      var nTotalSSDs = oSSDs.length;
      var nTotalSmartArraySSDs = 0;
      var nTotalNonSmartArraySSDs = 0;
      var nTotalSASSSDs = 0;
      var nTotalSATASSDs = 0;
      var bHasWearout = nTotalWearouts > 0;
      var bHas56Days = oSSD56DaysList.length > 0;
      var bHas2Percent = oSSD2PercentList.length > 0;
      var bHas5Percent = oSSD5PercentList.length > 0;

      // Div containing summary content.
      var s = '<div id="">';

      // Calculate totals
      for (var i = 0; i < nTotalSSDs; ++i)
      {
         var sSSDType = oSSDs[i].oDevice.deviceType;
         var sSSDInterface;
         if (IsDefined(sSSDType))
         {
            if (sSSDType == "NonSmartArrayPhysicalDrive")
            {
               ++nTotalNonSmartArraySSDs;
               sSSDInterface = this.getValue(oSSDs[i].oDevice.PhysicalDriveInfo, "Interface");
            }
            else
            {
               ++nTotalSmartArraySSDs;
               var interfaceValue = this.getValue(oSSDs[i].oDevice.PhysicalDriveStatus, "Device Type");
               if (interfaceValue == "0x02")
               {
                  sSSDInterface = "SAS_SSD"
               }
               else
               {
                  sSSDInterface = "SATA_SSD"
               }
            }
         }
         if (IsDefined(sSSDInterface))
         {
            if (sSSDInterface == "SAS_SSD")
            {
               ++nTotalSASSSDs;
            }
            else if (sSSDInterface == "SATA_SSD")
            {
               ++nTotalSATASSDs;
            }
         }
      }

      var oDeviceDetailsDiv = $('<div/>', {'class': 'deviceDetails'})
      var oDetailsDiv = $('<div/>', {'class': 'detailsDiv'});

      var oSectionsUL = $('<ul/>', {'class': 'sections'});
      var sSectionName = "STR_SUMMARY";
   
      var oSectionLI = $('<li/>', {'class': 'section'});
      var oSectionNameDiv = $('<div/>', {'class': 'sectionName'});
      oSectionNameDiv.html(TranslateString(sSectionName));
      oSectionLI.append(oSectionNameDiv);
      var oSectionDetailsTable = $('<table/>', {'class': 'sectionDetails'});

      // Total SSDs with Wearout
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_WEAROUTS", nTotalWearouts);

      // Total SA SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_SA_SSDS", nTotalSmartArraySSDs);

      // Total Non-SA SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_NON_SA_SSDS", nTotalNonSmartArraySSDs);

      // Total SATA SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_SATA_SSDS", nTotalSATASSDs);

      // Total SAS SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_SAS_SSDS", nTotalSASSSDs);

      // Total SSDs
      this.drawStringPair(oSectionDetailsTable, "STR_TOTAL_SSDS", nTotalSSDs);

      oSectionLI.append(oSectionDetailsTable);
      oSectionsUL.append(oSectionLI);
      oDetailsDiv.append(oSectionsUL);
      oDeviceDetailsDiv.append(oDetailsDiv);
      oBox.append(oDeviceDetailsDiv);

      // List SSDs with wearout if any exists.
      if (bHasWearout)
      {
         this.drawStatusTable(
            oSectionsUL,
            "criticalActive",
            "STR_SSD_WITH_SMART_TRIP_WEAROUT",
            "STR_MSG_SSDS_SMART_TRIP",
            oSSDWearoutList);
      }

      // Table: SSDs with 56 days warning.
      if (oSSD56DaysList.length > 0)
      {
         this.drawStatusTable(
            oSectionsUL,
            "infoActive",
            "STR_SSD_WITH_56DAY_WARNING",
            "STR_MSG_SSDS_56DAY_WARNING",
            oSSD56DaysList);
      }

      // Table: SSDs with 2% usage remaining warning.
      if (oSSD2PercentList.length > 0)
      {
         this.drawStatusTable(
            oSectionsUL,
            "infoActive",
            "STR_SSD_WITH_2PERCENT_WARNING",
            "STR_MSG_SSDS_2PERCENT_WARNING",
            oSSD2PercentList);
      }

      // Table: SSDs with 5% usage remaining warning.
      if (oSSD5PercentList.length > 0)
      {
         this.drawStatusTable(
            oSectionsUL,
            "infoActive",
            "STR_SSD_WITH_5PERCENT_WARNING",
            "STR_MSG_SSDS_5PERCENT_WARNING",
            oSSD5PercentList);
      }
   },

   drawStatusTable: function(oParentDiv, sIconClass, sSectionName, sNote, oDevices) {
      var oSectionLI = $('<li/>', {'class': 'section'});
      var oSectionNameDiv = $('<div/>', {'class': 'sectionName'});
      oSectionNameDiv.html(TranslateString(sSectionName));
      oSectionLI.append(oSectionNameDiv);
      var oSectionDetailsTable = $('<table/>', {'class': 'sectionDetails'});
      // Draw the note
      var sLabel = TranslateString(sNote);
      var oTR = $('<tr/>', {'class': ''});
      var oLabelTD = $('<td/>', {'class': 'label'});
      oLabelTD.html(TranslateString(sLabel));
      oTR.append(oLabelTD);
      oSectionDetailsTable.append(oTR);

      // Draw each "interesting" item
      for (var i=0; i<oDevices.length; ++i)
      {
         var sLabel = TranslateString("STR_SSD_WEAR_GAUGE_LABEL|" +
                                       oDevices[i].nSize + "|" +
                                       oDevices[i].sInterfaceShort +  "|" +
                                       oDevices[i].oController.marketingName + "|" +
                                       oDevices[i].sLocation);
         var oTR = $('<tr/>', {'class': ''});
         var oLabelTD = $('<td/>', {
            'class': 'data SSDLink',
            'id': oDevices[i].oDevice.id
         });
         oLabelTD.html(TranslateString(sLabel));
         oTR.append(oLabelTD);
         oSectionDetailsTable.append(oTR);
      }
      oSectionLI.append(oSectionDetailsTable);
      oParentDiv.append(oSectionLI);
   },

   clickOverviewButton: function(e) {
      this.model.set({'sDeviceID':''});
      this.drawAll();
   },

   getSSDs: function(sAttrName, sAttrValue) {
      if (IsDefined(sAttrName) && IsDefined(sAttrValue))
      {
         var oFoundSSDs = [];
         var oSSDDevices = this.model.get("oSSDDevices");
         for (var i = 0; i < oSSDDevices.length; ++i)
         {
            if (this.getValue(oSSDDevices[i].oDevice.SmartSSDWearGauge, sAttrName) == sAttrValue)
            {
               oFoundSSDs[oFoundSSDs.length] = oSSDDevices[i];
            }
         }
         return oFoundSSDs;
      }
      else
      {
         return oSSDDevices;
      }
   },

   drawDetailsWidget: function(oParentDiv) {
      var oWidgetDiv = this.drawColumnWidget(oParentDiv, "detailsWidget");
      var oBox = this.drawBox(oWidgetDiv);
      var sDeviceID = this.model.get("sDeviceID");
      var oSSDDevices = this.model.get("oSSDDevices");
      for (var i=0; i<oSSDDevices.length; ++i)
      {
         if (oSSDDevices[i].oDevice.id == sDeviceID)
         {
            var oSSD = oSSDDevices[i];
            var oDevice = oSSDDevices[i].oDevice;
         }
      }

      // Draw the SmartSSD Status Information
      // Begin the table
      var oDeviceDetailsDiv = $('<div/>', {'class': 'deviceDetails'})
      var oDetailsDiv = $('<div/>', {'class': 'detailsDiv'});
      var oSectionsUL = $('<ul/>', {'class': 'sections'});
   
      // Display SSD Wear Status
      this.drawDetailSection(oSSD, oDevice, oSectionsUL, "STR_SMARTSSD_STATUS");

      // Get info for physical drive details
      var sSSDType = oDevice.deviceType;
      var sPhysicalDrive;
      if (IsDefined(sSSDType))
      {
         if (sSSDType == "NonSmartArrayPhysicalDrive")
         {
            sPhysicalDrive = "PhysicalDriveInfo";
         }
         else
         {
            sPhysicalDrive = "PhysicalDriveStatus";
         }

         // Display Physical Drive Table
         this.drawDetailSection(oSSD, oDevice, oSectionsUL, "STR_PHYSICAL_DRIVE", sPhysicalDrive);

         // Display Physical Drive Authentication Table
         var oController = oSSD.oController;
         if (IsDefinedAndSet(oController))
         {
            var sCtrlAuthSupport = this.getValue(oController.IdentifyController, "Even More Controller Flags");
            var bCtrlAuthSupport = false;
            if (IsDefinedAndSet(sCtrlAuthSupport))
            {
               bCtrlAuthSupport = sCtrlAuthSupport & 0x4000;
            }
            if (bCtrlAuthSupport)
            {
               //this.drawDetailSection(oSSD, oDevice, oSectionsUL, "STR_PHYSICAL_DRIVE_AUTHENTICATION", sPhysicalDrive);
            }
         }

         // Display Location Table
         this.drawDetailSection(oSSD, oDevice, oSectionsUL, "STR_LOCATION", sPhysicalDrive);
      }

      oDetailsDiv.append(oSectionsUL);
      oDeviceDetailsDiv.append(oDetailsDiv);
      oBox.append(oDeviceDetailsDiv);
   },

   drawDetailSection: function(oSSD, oDevice, oParentDiv, sSectionName, sPhysicalDrive) {
      var oSectionLI = $('<li/>', {'class': 'section'});
      var oSectionNameDiv = $('<div/>', {'class': 'sectionName'});
      oSectionNameDiv.html(TranslateString(sSectionName));
      oSectionLI.append(oSectionNameDiv);
      var oSectionDetailsTable = $('<table/>', {'class': 'sectionDetails'});
      var bDrawSection = true;

      if (sSectionName == "STR_SMARTSSD_STATUS")
      {
         // Gather initial info
         var sSmartSSDLogFull = this.getValue(oDevice.SmartSSDWearGauge, TranslateString("STR_LOG_FULL"));
         var bSmartSSDLogFull = IsDefined(sSmartSSDLogFull) && sSmartSSDLogFull == "TRUE";
         var bValidData = oSSD.bSmartSSDSupported && !bSmartSSDLogFull;

         // Get utilization information
         if (bValidData)
         {
            var nUtilization;
            var sUtilization = this.getValue(oDevice.SmartSSDWearGauge, "Utilization");
            var nLifeLeft = oSSD.nUsageRemaining;
            var sDaysRemaining = oSSD.sDaysRemaining;
            var sSmartTrip = this.getValue(oDevice.SmartSSDWearGauge, "SSD SMART Trip for Wearout");
            var bValidLifeLeft = IsDefined(sUtilization) && sUtilization >= 0 && sUtilization <= 100;
            if (bValidLifeLeft)
            {
               nUtilization = parseFloat(sUtilization);
               nLifeLeft = 100 - nUtilization;
               nLifeLeft = Math.round(nLifeLeft*100)/100;
            }
         }

         var sSSDWearStatus = oSSD.sStatusText;
         if (!oSSD.bSmartSSDSupported)
         {
            sSSDWearStatus = "STR_NOT_SUPPORTED";
         }
         else if (bSmartSSDLogFull)
         {
            sSSDWearStatus = "STR_LOG_FULL_INFO";
         }
         this.drawStringPair(oSectionDetailsTable, "STR_SSD_WEAR_STATUS", TranslateString(sSSDWearStatus));

         if (bValidData)
         {
            var sPowerOnHours = this.getValue(oDevice.SmartSSDWearGauge, "STR_POWER_ON_HOURS");
            this.drawStringPair(oSectionDetailsTable, "STR_POWER_ON_HOURS", sPowerOnHours);
            if (IsDefinedAndSet(sDaysRemaining))
            {
               this.drawStringPair(oSectionDetailsTable, "STR_SSD_DAYS_REMAINING", TranslateString("STR_DAYS|" + sDaysRemaining));
            }
            if (bValidLifeLeft)
            {
               this.drawStringPair(oSectionDetailsTable, "STR_LIFE_REMAINING", TranslateString("STR_PERCENT|" + nLifeLeft));
               this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_EXINFO_SSD_UTILIZATION", TranslateString("STR_PERCENT|" + nUtilization));
            }
         }
      }
      else if (sSectionName == "STR_PHYSICAL_DRIVE")
      {
         var sSerialNumber, sFirmwareRevision;
         if (sPhysicalDrive == "PhysicalDriveInfo")
         {
            oPhysicalDrive = oDevice.PhysicalDriveInfo;
            sSerialNumber = this.getValue(oPhysicalDrive, "SerialNumber");
            sFirmwareRevision = this.getValue(oPhysicalDrive, "Firmware Revision");
         }
         else
         {
            oPhysicalDrive = oDevice.PhysicalDriveStatus;
            sSerialNumber = this.getValue(oPhysicalDrive, "Drive Serial Number");
            sFirmwareRevision = this.getValue(oPhysicalDrive, "Drive Firmware Revision");
         }
         // Display Status
         var sPhysicalStatus = oSSD.sPhysicalStatus;
         if (IsDefined(sPhysicalStatus))
         {
            this.drawStringPair(oSectionDetailsTable, "STR_PHYSICAL_DRIVE_STATUS", sPhysicalStatus);
         }
         var sSize = oSSD.nSize;
         if (IsDefined(sSize))
         {
            this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_TOTAL_BLOCKS", sSize);
         }
         var sSSDInterface = oSSD.sInterface;
         if (IsDefined(sSSDInterface))
         {
            this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_LOGICAL_DRIVE_TYPE", sSSDInterface);
         }
         this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "Drive Model", "ATTR_NAME_DRIVE_MODEL");

         if (IsDefined(sSerialNumber))
         {
            this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_SERIAL_NUMBER", sSerialNumber);
         }
         if (IsDefined(sFirmwareRevision))
         {
            this.drawStringPair(oSectionDetailsTable, "STR_FIRMWARE_REVISION", sFirmwareRevision);
         }
         //this.DisplayTemp("Current Temperature", oPhysicalDrive);
         //this.DisplayTemp("Maximum Temperature", oPhysicalDrive);
         this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "ATTR_NAME_CURRENT_TEMPERATURE");
         this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "ATTR_NAME_MAXIMUM_TEMPERATURE");
         //this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "GUI_NAME_PHY_COUNT");
         //this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "ATTR_NAME_NEGOTIATED_LINK_RATE", "GUI_NAME_TRANSFER_SPEED");
      }
      else if (sSectionName == "STR_PHYSICAL_DRIVE_AUTHENTICATION")
      {
         if (sPhysicalDrive == "PhysicalDriveInfo")
         {
            oPhysicalDrive = oDevice.PhysicalDriveInfo;
         }
         else
         {
            oPhysicalDrive = oDevice.PhysicalDriveStatus;
         }
         var sDriveState = this.getValue(oPhysicalDrive, "STR_DRIVE_AUTENTICATION");
         var bDriveAuthSupported = IsDefinedAndSet(sDriveState);
         var sCarrierState = this.getValue(oPhysicalDrive, "STR_SMART_CARRIER_AUTHENTICATION");
         var bCarrierAuthSupported = IsDefinedAndSet(sCarrierState);
         if (!bCarrierAuthSupported || sCarrierState == 0)
         {
            sCarrierState = "STR_NOT_APPLICABLE";
            // Drive authentication is not applicable if carrier 
            // authentication is not applicable.
            sDriveState = "STR_NOT_APPLICABLE";
         }
         else
         {
            switch (sCarrierState)
            {
               case 1:
                  sCarrierState = "STR_OK";
                  break;
               case 2:
                  sCarrierState = "STR_AUTHENTICATION_FAILED_COMMUNICATION";
                  break;
               case 255:
               default:
                  sCarrierState = "STR_AUTHENTICATION_FAILED_NOT_HP";
                  break;
            }

            if (!bDriveAuthSupported)
            {
               sDriveState = "STR_NOT_APPLICABLE";
            }
            else
            {
               switch (sDriveState)
               {
                  case 0:
                     sDriveState = "STR_AUTHENTICATION_FAILED_FIRST";
                     break;
                  case 1:
                     sDriveState = "STR_AUTHENTICATION_FAILED_SECOND";
                     break;
                  case 0x80:
                     sDriveState = "STR_OK";
                     break;
                  default:
                     sDriveState = "STR_AUTHENTICATION_FAILED_ONE";
                     break;
               }
            }
         }
         // Only show the drive authentication state if it's applicable.
         if (sDriveState != "Not Applicable")
         {
            this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_CARRIER_AUTHENTICATION_STATUS", TranslateString(sDriveState));
         }
         // If controller supports drive and carrier authentication, always 
         // show the carrier authentication state, even when not applicable.
         this.drawStringPair(oSectionDetailsTable, "STR_DRIVE_CARRIER_AUTHENTICATION_STATUS", TranslateString(sCarrierState));
         // The two version fields will only contain meaningful data if 
         // the carrier has authenticated successfully (it's in OK state).
         if (sCarrierState == "OK")
         {
            this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "SMART_CARRIER_APPLICATION_VERSION");
            this.drawAttribute(oSectionDetailsTable, oPhysicalDrive, "SMART_CARRIER_BOOTLOADER_VERSION");
         }
      }
      else if (sSectionName == "STR_LOCATION")
      {
         if (sPhysicalDrive == "PhysicalDriveInfo")
         {
            oPhysicalDrive = oDevice.PhysicalDriveInfo;
         }
         else
         {
            oPhysicalDrive = oDevice.PhysicalDriveStatus;
         }
         // Location
         var sAttrBox = this.getValue(oPhysicalDrive, "Physical Box");
         var sAttrBay = this.getValue(oPhysicalDrive, "Physical Bay");
         var sAttrPort = this.getValue(oPhysicalDrive, "Physical Port");
         if (IsDefined(sAttrBox))
         {
            // Convert to int
            sAttrBox = parseInt(sAttrBox);
            sAttrBay = parseInt(sAttrBay);

            // Display Port
            if (IsDefined(sAttrPort))
            {
               this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_PORT", sAttrPort);
            }
            // Display Box
            if (sAttrBox == "0")
            {
               this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_BOX", TranslateString("STR_ZERO_NOT_APPLICABLE"));
            }
            else
            {
               this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_BOX", sAttrBox);
            }
            // Display Bay
            if (IsDefined(sAttrBay))
            {
               if (sAttrBay == "0" && sAttrBox == "0")
               {
                  this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_BAY", TranslateString("STR_ZERO_NOT_APPLICABLE"));
               }
               else
               {
                  this.drawStringPair(oSectionDetailsTable, "ATTR_NAME_BAY", sAttrBay);
               }
            }
            // Display Location Hint
            /*var sLocHint = this.getValue(oPhysicalDrive, "Location Hint");
            if (IsDefined(sLocHint) && sLocHint != "None")
            {
               this.DisplayAttr("Location Hint");
            }*/
         }
         else
         {
            bDrawSection = false;
         }
      }
      if ( bDrawSection )
      {
         oSectionLI.append(oSectionDetailsTable);
         oParentDiv.append(oSectionLI);
      }
   },

   drawStringPair: function(oParentDiv, sLabel, sData) {
      var oTR = $('<tr/>', {'class': ''});
      var oLabelTD = $('<td/>', {'class': 'label'});
      var oDataTD = $('<td/>', {'class': 'data'});
      oLabelTD.html(TranslateString(sLabel));
      oDataTD.html(sData);
      oTR.append(oLabelTD);
      oTR.append(oDataTD);
      oParentDiv.append(oTR);
   },

   getValue: function(o, sAttrName) {
      var sValue;
      if (IsDefined(o))
      {
         for (i=0; i<o.length; ++i)
         {
            if (o[i].id == TranslateString(sAttrName))
            {
               sValue = o[i].value;
            }
         }
      }
      return sValue;
   },

   drawAttribute: function(oParentDiv, oDevice, sAttrName, sDisplayName) {
      var bDrawAttribute = true;
      if (!IsDefined(sDisplayName))
      {
         var sDisplayName = TranslateString(sAttrName);
      }
      else {
         sDisplayName = TranslateString(sDisplayName);
      }
      var sValue = this.getValue(oDevice, sAttrName);
      if (sAttrName == "GUI_NAME_PHY_COUNT")
      {
         sValue = parseInt(sValue, 16);
      }
      else if (sAttrName == "ATTR_NAME_NEGOTIATED_LINK_RATE") 
      {
         sValue = parseInt(sValue, 16);
      }
      else if (sAttrName == "ATTR_NAME_MAXIMUM_TEMPERATURE" ||
               sAttrName == "ATTR_NAME_CURRENT_TEMPERATURE")
      {  
         sValue = parseInt(sValue, 16);
         if ( isNaN(sValue) )
         {
            bDrawAttribute = false;
         }
         sValue = sValue + "&deg;C";
      }
      if ( bDrawAttribute )
      {
         this.drawStringPair(oParentDiv, sDisplayName, sValue);
      }
   },

   drawFooter: function() {
      var oOpFooterDiv = $('<div/>', {
         'class':"opFooter"
      });
      var oInfoDiv = $('<div/>', {
         'class':"info"
      });
      var oLoadingDiv = $('<div/>', {
         'class':"loading"
      });
      var oButtonsDiv = $('<div/>', {
         'class':"buttons"
      });
      oOpFooterDiv.append(oInfoDiv);
      oOpFooterDiv.append(oLoadingDiv);
      oOpFooterDiv.append(oButtonsDiv);
      $(this.el).append(oOpFooterDiv);
   },

   drawDetailedStatus: function() {
      this.model.set({'sDeviceID':'status'});
      this.drawAll();
      return false;
   }
   
});