var VHelpDisplay = BaseView.extend({

   initialize: function() {
      BaseView.prototype.initialize.call(this);
      this.model.bind("change:sTitle", this.updateDisplay, this);
      this.render();
   },
   
   render: function() {
      var oDisplayDiv = $('<div/>', {'class':'helpDisplay'});
      this.$el.append(oDisplayDiv);
      this.drawHeader();
      this.drawContent();
      this.updateDisplay();
      return this;
   },

   drawHeader: function() {
      var oDisplayDiv = this.$('.helpDisplay');
      var oItemHeader = $('<div/>', {'class':'helpItemHeader'});
      oDisplayDiv.append(oItemHeader);
   },
   
   drawContent: function() {
      var oDisplayDiv = this.$('.helpDisplay');
      var oItemBody = $('<div/>', {'class':'helpItemBody'});
      oDisplayDiv.append(oItemBody);
   },

   updateDisplay: function() {
      var oThis = this;
      var sTitle = oThis.model.get('sTitle');
      // Update the header
      var oItemHeader = oThis.$('.helpItemHeader');
      oItemHeader.empty();
      oItemHeader.html(TranslateString(oThis.model.get('sTitle')));

      // Draw help content
      var oHelpContent = oThis.model.get('oHelpContent');
      var oItemBody = oThis.$('.helpItemBody');
      oItemBody.empty();
      $.each(oHelpContent, function(key, val) {
         oItemBody.append(oThis.drawItemSection(key));
         oThis.drawItemBody(val, oItemBody);
      })
   },

   drawItemSection: function(key) {
      var h = $('<div/>', {
         'class': 'helpSubHeader'
      });
      h.html(TranslateString(key));
      return h;
   },

   drawItemBody: function(oContent, oDiv) {
      var oThis = this;
      $.each(oContent, function(key, val) {
         var type = key.replace(/\d+/g, '');
         if (type == "body") {
            for (var i=0; i<val.length; i++) {
               var p = $('<p/>');
               var sContent = oThis.addInlineLinks(TranslateString(val[i]));
               p.html(sContent);
               oDiv.append(p);   
            }
         }
         else if (type == "note") {
            for (var i=0; i<val.length; i++) {
               var n = $('<a/>', {
                  'class':'helpNote'
               });
               n.html(TranslateString("HELP_NOTE") + ": ");
               var p = $('<p/>');
               var sContent = oThis.addInlineLinks(TranslateString(val[i]));
               p.html(sContent);
               p.prepend(n);
               oDiv.append(p);
            }
         }
         else if (type == "task") {
            var oTaskWrapper = $('<div/>', {
               'class':'helpTask'
            });
            $.each(val[0], function(task, desc) {
               var oTaskItemWrapper = $('<div/>', {
               'class':'helpTaskItem'
               });
               var oTaskName = $('<div/>', {
                  'class':'helpTaskName'
               });
               var oTaskDesc = $('<div/>', {
                  'class':'helpTaskDesc'
               });
               oTaskItemWrapper.append(oTaskName);
               oTaskItemWrapper.append(oTaskDesc);
               oTaskName.html(TranslateString(task));
               var sTaskDesc = oThis.addInlineLinks(TranslateString(desc));
               oTaskDesc.html(sTaskDesc);
               oTaskItemWrapper.append(GetClearFloatsDiv());
               oTaskWrapper.append(oTaskItemWrapper);
            })
            oDiv.append(oTaskWrapper);
         }
         else if (type == "table") {
            // val[0] is a title string
            // val[1] is a header key-value pair
            // val[2] is an object of data key-value pairs
            var oTable = $('<table/>', {
               'class': 'helpTable'
            });
            // Create the Title
            var oTitleTR = $('<tr/>');
            var oTitleTH = $('<th/>', {
               'class': 'title',
               'colspan': '2'
            });
            oTitleTH.html(TranslateString(val[0]));
            // Create the Headers
            var oHeaderTR = $('<tr/>');
            var oHeaderTH1 = $('<th/>', {
               'class': 'header1'
            });
            var oHeaderTH2 = $('<th/>', {
               'class': 'header2'
            });
            for (tableItem in val[1]) {
               oHeaderTH1.html(TranslateString(tableItem));
               oHeaderTH2.html(TranslateString(val[1][tableItem]));
            }
            oTable.append(oTitleTR);
            oTitleTR.append(oTitleTH);
            oTable.append(oHeaderTR);
            oHeaderTR.append(oHeaderTH1);
            oHeaderTR.append(oHeaderTH2);

            // Create the Data cells

            $.each(val[2], function(tableItem, val) {
               var oTR = $('<tr/>');
               var oTD1 = $('<td/>');
               var oTD2 = $('<td/>');
               oTD1.html(TranslateString(tableItem));
               oTD2.html(TranslateString(val));
               oTR.append(oTD1);
               oTR.append(oTD2);
               oTable.append(oTR);
            })
            oDiv.append(oTable);
         }
         else if (type == "list") {
            var oUL = $('<ul/>', {
               'class': 'helpList'
            });
            for (var i=0; i<val.length; ++i) {
               var oLI = $('<li/>');
               var sContent = oThis.addInlineLinks(TranslateString(val[i]));
               oLI.html(sContent);
               oUL.append(oLI);
            }
            oDiv.append(oUL);
         }
         else if (type == "legend") {
            var oTable = $('<table/>', {
               'class': 'helpLegendTable'
            });
            // Create the Headers
            var oHeaderTR = $('<tr/>');
            var oHeaderTH1 = $('<th/>', {
               'class': 'header1'
            });
            var oHeaderTH2 = $('<th/>', {
               'class': 'header2'
            });
            oHeaderTH1.html(TranslateString("STR_IMAGE"));
            oHeaderTH2.html(TranslateString("STR_DESCRIPTION"));
            oTable.append(oHeaderTR);
            oHeaderTR.append(oHeaderTH1);
            oHeaderTR.append(oHeaderTH2);
            // Create the Data cells
            oThis.drawHelpImageLegend(oTable);
            oDiv.append(oTable);
         }
         else if (type == "weargaugelegend") {
            oThis.drawWearGaugeLegendItem("green", oDiv);
            oThis.drawWearGaugeLegendItem("blue", oDiv);
            oThis.drawWearGaugeLegendItem("yellow", oDiv);
            oThis.drawWearGaugeLegendItem("red", oDiv);
         }
      })
   },

   drawHelpImageLegend: function(oTable) {
      this.drawHelpImageLegendItem("ATTR_VALUE_STATUS_CRITICAL", "statusCritical", oTable);
      this.drawHelpImageLegendItem("GUI_VALUE_STATUS_TYPE_WARNING", "statusWarn", oTable);
      this.drawHelpImageLegendItem("HELP_STATUS_ALERTS_INFORMATIONAL", "statusInfo", oTable);
      this.drawHelpImageLegendItem("STR_GUI_ACTIVE_TASKS", "statusTask", oTable);
      this.drawHelpImageLegendItem("STR_HP_PROLIANT_SERVER", "proliant", oTable);
      this.drawHelpImageLegendItem("STR_ARRAY_CONTROLLER", "arrayController", oTable);
      this.drawHelpImageLegendItem("STR_ARRAY_CONTROLLER_EMBEDDED", "arrayControllerEmbedded", oTable);
      //this.drawHelpImageLegendItem("STR_FAILED_ARRAY_CONTROLLER", "arrayControllerFailed", oTable);
      //this.drawHelpImageLegendItem("STR_FAILED_ARRAY_CONTROLLER_EMBEDDED", "arrayControllerEmbeddedFailed", oTable);
      this.drawHelpImageLegendItem("STR_ARRAY", "arrayIcon", oTable);
      this.drawHelpImageLegendItem("STR_LOGICAL_DRIVE", "logicalDrive", oTable);
      this.drawHelpImageLegendItem("STR_ASSIGNED_PHYSICAL_DRIVE", "assignedPhysicalDrive", oTable);
      this.drawHelpImageLegendItem("STR_UNASSIGNED_PHYSICAL_DRIVE", "unassignedPhysicalDrive", oTable);
      this.drawHelpImageLegendItem("STR_UNASSIGNED_DRIVES", "unassignedDrives", oTable);
      this.drawHelpImageLegendItem("STR_TAPE_DRIVE", "tapeDrive", oTable);
      this.drawHelpImageLegendItem("STR_SPARE_DRIVE", "spareDrive", oTable);
      this.drawHelpImageLegendItem("STR_TRANSIENT_DRIVE", "transientDrive", oTable);
      this.drawHelpImageLegendItem("STR_PAUSED_OFFLINE", "pausedOfflineDrive", oTable);
      this.drawHelpImageLegendItem("STR_LOCKED", "locked", oTable);
      this.drawHelpImageLegendItem("STR_CACHE_MANAGER", "cacheManager", oTable);
      this.drawHelpImageLegendItem("STR_LICENSE_MANAGER", "licenseManager", oTable);
      this.drawHelpImageLegendItem("ATTR_VALUE_TYPE_STORAGE_ENCLOSURE", "storageEnclosure", oTable);
      //this.drawHelpImageLegendItem("STR_ENCRYPTION_MANAGER", "encryptionManager", oTable);
   },

   drawHelpImageLegendItem: function(sDesc, sIcon, oTable) {
      var oTR = $('<tr/>');
      var oTD1 = $('<td/>');
      var oTD2 = $('<td/>');
      var oIconDiv = $('<div/>', {
         'class': 'helpImageLegendIcon HPSSA_SPRITE ' + sIcon
      });
      oTD1.append(oIconDiv);
      oTD2.html(TranslateString(sDesc));
      oTR.append(oTD1);
      oTR.append(oTD2);
      oTable.append(oTR);
   },

   drawWearGaugeLegendItem: function(sColor, oDiv) {
      var p = $('<p/>');
      var sHelpString = '';
      var sIcon = '';
      var sWearGaugeStatus = '';
      var nSSDWear = 100;
      if (sColor == "green") {
         sHelpString = "HELP_SMARTSSD_IMAGE_LEGEND_GREEN";
         sWearGaugeStatus = "wearGaugeOK";
         nSSDWear = 95;
      }
      else if (sColor == "blue") {
         sHelpString = "HELP_SMARTSSD_IMAGE_LEGEND_BLUE";
         sWearGaugeStatus = "wearGaugeInfo";
         nSSDWear = 10;
         sIcon = 'infoActive';
      }
      else if (sColor == "yellow") {
         sHelpString = "HELP_SMARTSSD_IMAGE_LEGEND_YELLOW";
         sWearGaugeStatus = "wearGaugeTripped";
         nSSDWear = 2;
         sIcon = 'warnActive';
      }
      else if (sColor == "red") {
         sHelpString = "HELP_SMARTSSD_IMAGE_LEGEND_RED";
         sWearGaugeStatus = "wearGaugeFailed";
         nSSDWear = 2;
         sIcon = 'criticalActive';
      }
      var sContent = this.addInlineLinks(TranslateString(sHelpString));
      p.html(sContent);
      var oDummyBar = this.drawDummyBar(sWearGaugeStatus, nSSDWear);
      oDiv.append(p);
      oDiv.append(oDummyBar);
      if (sIcon != '') {
         var oIconDiv = $('<div/>', {
            'class': 'helpWearGaugeLegendIcon HPSSA_SPRITE ' + sIcon
         });
         oDiv.append(oIconDiv);
      }
      oDiv.append(GetClearFloatsDiv());
   },

   drawDummyBar: function(sWearGaugeStatus, nSSDWear) {
      var sGaugeDisplayFull = nSSDWear + "%";
      var sGaugeDisplayEmpty = 100 - nSSDWear + "%";
      // Build the bar
      var oWearGaugeBarDiv = $('<div/>', {'class': 'wearGaugeBarWrapper'});
      var oWearGaugeBar = $('<table/>', {
         'class':'wearGaugeBarTable'
      });
      var oWearGaugeBarTR = $('<tr/>', {
         'class':'wearGaugeBarTR'
      });
      var oWearGaugeBarDone = $('<td/>', {
         'id': 'wearGaugeFull',
         'class':sWearGaugeStatus,
         'width':sGaugeDisplayFull
      });
      var oWearGaugeBarNotDone = $('<td/>', {
         'id': 'wearGaugeEmpty',
         'class':'wearGaugeEmpty',
         'width':sGaugeDisplayEmpty
      });
      oWearGaugeBarTR.append(oWearGaugeBarDone);
      oWearGaugeBarTR.append(oWearGaugeBarNotDone);
      oWearGaugeBar.append(oWearGaugeBarTR);
      oWearGaugeBarDiv.append(oWearGaugeBar);
      return oWearGaugeBarDiv;
   },

   addInlineLinks: function(sContent) {
      var sCurrentPage = this.model.get("sTitle");
      var oInlineLinks = this.model.get("oInlineLinks");
      var sContentWithLinks = sContent;
      var foundInlineLinks = [];
      var foundIndices = [];
      for (var j=0; j<oInlineLinks.length; ++j) {
         // Do not add inline links to self
         var sLinkName = oInlineLinks[j].sLinkName;
         var helpIndexNumber = oInlineLinks[j].helpIndexNumber;
         var sLinkText = TranslateString(sLinkName);
         if (sLinkName != sCurrentPage) {
            // Search for the linkName in the content
            var sContentCopy = sContent;
            var oLinkInfo = oInlineLinks[j].oLinkInfo;
            var searchResult = this.findLinkName(sLinkText, sContentCopy, oLinkInfo.isCaseSensitive, oLinkInfo.exceptions);
            if (searchResult != -1) {
               // See if longer form of the search phrase exists at same location.
               // Do this for English only
               var pluralFound = false;
               var sPluralLinkText = this.pluralizeName(sLinkText);
               pluralFound = searchResult == this.findLinkName(sPluralLinkText, sContentCopy, oLinkInfo.isCaseSensitive, oLinkInfo.exceptions);
               if(pluralFound)
               {
                  sLinkText = sPluralLinkText;
               }
               // Get the link text from the content to preserve capitalization
               sLinkText = sContent.substr(searchResult, sLinkText.length);
               if(!IsDefined(foundInlineLinks[searchResult]))
               {
                  // Save index.
                  foundIndices[foundIndices.length] = searchResult;
                  // Save search string.
                  // If plural match was found, copy inline link without
                  // the exceptions.
                  foundInlineLinks[searchResult] = {
                     "sLinkName":sLinkName,
                     "isDefinition":oLinkInfo.isDefinition,
                     "isCaseSensitive":oLinkInfo.isCaseSensitive,
                     "sLinkText":sLinkText,
                     "helpIndexNumber":helpIndexNumber
                  };
               }
               // Save search string only if it is longer than the previous search
               // string that matched.
               else if (sLinkText.length > foundInlineLinks[searchResult].sLinkText.length)
               {
                  // Overwrite the old search string with the new one.
                  foundInlineLinks[searchResult] = {
                     "sLinkName":sLinkName,
                     "isDefinition":oLinkInfo.isDefinition,
                     "isCaseSensitive":oLinkInfo.isCaseSensitive,
                     "sLinkText":sLinkText,
                     "helpIndexNumber":helpIndexNumber
                  };
               }
            }
         }
      }
      foundIndices.sort(this.sortNumber);
      var prevPosEnd = -1;
      // Now make sure no search string runs into another. If one does, only keep
      // the string that came first in the content.
      for (var j=0; j < foundIndices.length; ++j)
      {
         var currPosBegin = foundIndices[j];
         // Check if current string runs into previous string.
         if(currPosBegin <= prevPosEnd)
         {
            // It does, so remove the current string since the previous came first
            // in the content.
            delete foundInlineLinks[currPosBegin];
            delete foundIndices[j];
         }
         else
         {
            // It does not, so we can safely advance the end position that we want
            // to make sure no other search strings touch.
            prevPosEnd = foundInlineLinks[currPosBegin].sLinkText.length + 
                         currPosBegin - 1;
         }
      }

      for (var revIndex = foundIndices.length - 1; revIndex >= 0; --revIndex)
      {
         // foundIndices contains deleted entries, so only look at defined ones.
         // foundInlineLinks only contains entries that are mapped by
         // foundIndices, so no need to check if defined.
         if(IsDefined(foundIndices[revIndex]))
         {
            var posBegin = foundIndices[revIndex];
            var posEnd = foundInlineLinks[posBegin].sLinkText.length + posBegin;
            sContentWithLinks = sContentWithLinks.substr(0, posBegin) +
                                  this.getInlineLink(foundInlineLinks[posBegin])/*.prop('outerHTML')*/ +
                                  sContentWithLinks.substr(posEnd);
         }
      }
      return sContentWithLinks;
   },

   findLinkName: function(sLinkName, sContent, isCaseSensitive, oExceptions) {
      if (!isCaseSensitive) {
         sLinkName = sLinkName.toLowerCase();
         sContent = sContent.toLowerCase();
      }
      var searchResult = sContent.indexOf(sLinkName);
      if (oExceptions.length > 0 && searchResult != -1) {
         for (var k=0; k<oExceptions.length; ++k)
         {
            var sException = TranslateString(oExceptions[k]);
            if (!isCaseSensitive) {
               sException = sException.toLowerCase();
            }
            // If exception found, then mark search result as not found.
            var exceptionSearchResult = sContent.indexOf(sException);
            if (exceptionSearchResult != -1) {
               if (searchResult == exceptionSearchResult || exceptionSearchResult == (searchResult - sException.length + sLinkName.length)) {
                  searchResult = -1;
                  break;
               }
            }
            
         }
      }
      return searchResult;
   },

   pluralizeName: function(sLinkName) {
      var sPluralLinkName = sLinkName;
      var sSuffix = "";
      var nNameLenth = sLinkName.length;
      if(nNameLenth > 0 && sLinkName.charAt(nNameLenth-1) == "s")
      {
         sSuffix = "es";
      }
      else
      {
         sSuffix = "s";
      }
      return sPluralLinkName + sSuffix;
   },

   sortNumber: function(a,b) {
      return a - b;
   },

   getInlineLink: function(oLinkInfo) {
      var oLinkWrapper = $('<div/>');
      var oLinkDiv = $('<a/>', {
         'href':'',
         'class':'inlineLink popupTabbable',
         'id':oLinkInfo.helpIndexNumber + 'inline'
      });
      oLinkDiv.html(oLinkInfo.sLinkText);
      oLinkWrapper.append(oLinkDiv);
      return oLinkWrapper.html();
   }
   
});