var VControlDeviceSelector = AVControl.extend({

   events: {
      "change select.filterBySelect" : "onFilterValueChange",
      "change select.groupBySelect" : "onGroupChange",
      "change select.sortBySelect" : "onSortByChange"
   },
   
   initialize: function() {
      var oFilter = this.model.get("oFilter");
      if (oFilter != false)
      {
         this.oUDFilter = new SALFilter(oFilter);
      }
      else
      {
         this.oUDFilter = false;
      }
      this.sFilterBySelectID = this.id + "filterBySelect";
      this.sFilterByLabelID = this.id + "filterByLabel";
      this.sGroupBySelectID = this.id + "groupBySelect";
      this.sGroupByLabelID = this.id + "groupByLabel";
      this.sSortBySelectID = this.id + "sortBySelect";
      this.sSortByLabelID = this.id + "sortByLabel";
      AVControl.prototype.initialize.call(this);
   },
   
   render: function() {
      if (this.oUDFilter != false && this.oUDFilter.isValidFilter())
      {
         var oFilterMenuDiv = $('<div/>', {'class': "filterMenu"});
         var oFilterGroupsDiv = $('<div/>', {'class': "filterGroups"});
         $(this.el).append(oFilterMenuDiv);
         $(this.el).append(oFilterGroupsDiv);
         this.drawMenu();
         this.drawDevices();
      }
      else
      {
         var oFilterGroupsDiv = $('<div/>', {'class': "filterGroups"});
         $(this.el).append(oFilterGroupsDiv);
         this.drawUnfilteredDevices();
      }
      return this;
   },
   
   getSelectedCount: function() {
      return this.getSelectedLI().length;
   },
   
   getSelectedLI: function() {
      return this.$('.filterGroups .ui-selected');
   },
   
   getValueArray: function() {
      var oSelected = this.getSelectedLI();
      for (var i=0; i<oSelected.length; ++i)
      {
         oSelected[i] = oSelected[i].id;
      }
      return oSelected;
   },
   
   getValue: function() {
      var s = '';
      var oSelected = this.getSelectedLI();
      for (var i=0; i<oSelected.length; ++i)
      {
         if (i>0)
         {
            s += '|';
         }
         s += oSelected[i].id;
      }
      return s;
   },
   
   drawMenu: function() {
      // Draw Filter Menu.
      var sPopupTabbableClass = this.model.get('bPopupTabbable') ? ' popupTabbable' : '';
      var oFilterMenuDiv = this.$('.filterMenu');
      var nNumFilterValues = this.oUDFilter.numFilterValues();
      if (nNumFilterValues > 0)
      {
         // Show a filter dropdown.
         var filterName = this.oUDFilter.filterName();
         
         // Add filter dropdown label.
         var oFilterByLabelSpan = $('<span/>', {
            'id': this.sFilterByLabelID,
            'class': 'label'
         });
         oFilterByLabelSpan.html(TranslateString(filterName));
         oFilterMenuDiv.append(oFilterByLabelSpan);

         // Add filter dropdown.
         var oFilterBySelect = $('<select/>', {
            'id': this.sFilterBySelectID,
            'class': 'filterBySelect' + sPopupTabbableClass
         });
         for ( var fvIndex = 0; fvIndex < nNumFilterValues; fvIndex++ )
         {
            this.oUDFilter.SelectFilterValue( fvIndex );
            var filterValueName = this.oUDFilter.filterValueName();		 
            var oOption = $('<option/>', {value: filterValueName});
            oOption.html(TranslateString(filterValueName));
            oFilterBySelect.append(oOption);
         }
         if (nNumFilterValues == 1)
         {
            // Hide Filter dropdown if there is only one option.
            oFilterByLabelSpan.hide();
            oFilterBySelect.hide();

            // Uncomment this and comment out the above if we want to 
            // show Filter menu as disabled and not hidden whenonly
            // one value exists.
            /*   
            if (filterName == "ATTR_NAME_ARRAY_TYPE")
            {
               oFilterByLabelSpan.hide();
               oFilterBySelect.hide();
            }
            else
            {
               oFilterBySelect.prop('disabled', 'disabled');
            }*/
         }
         oFilterMenuDiv.append(oFilterBySelect);
      }
      // Show the group by dropdown.
      this.oUDFilter.SelectFilterValue(0);
      
      // Add group by dropdown label.
      var oGroupByLabel = $('<span/>', {
         'id': this.sGroupByLabelID,
         'class': 'label'
      });
      oGroupByLabel.html(TranslateString("STR_GROUP_BY"));
      oFilterMenuDiv.append(oGroupByLabel);
      
      // Add group by dropdown.
      var oGroupBySelect = $('<select/>', {
         'id': this.sGroupBySelectID,
         'class': 'groupBySelect' + sPopupTabbableClass
      });
      var nNumGroups = this.oUDFilter.numGroups();
      // If more than one group by method, use the first non-"none" one.
      var nDefaultGroup = 0;
      if (nNumGroups > 1)
      {
         nDefaultGroup = 1;
      }
      for ( var gIndex = 0; gIndex < nNumGroups; ++gIndex )
      {
         this.oUDFilter.SelectGroup( gIndex );
         var groupName = this.oUDFilter.groupName();
         var oOption = $('<option/>', {value: groupName});
         oOption.html(TranslateString(groupName));
         if (gIndex == nDefaultGroup)
         {
            oOption.prop('selected', 'selected');
         }
         oGroupBySelect.append(oOption);
      }
      if (nNumGroups == 1)
      {
         // Hide Group By dropdown if there is only one option.
         oGroupByLabel.hide();
         oGroupBySelect.hide();

         //oGroupBySelect.prop('disabled', 'disabled');
      }
      oFilterMenuDiv.append(oGroupBySelect);
      
      // Show the sort by dropdown
      this.oUDFilter.SelectGroup( nDefaultGroup );
      this.oUDFilter.SelectGroupValue( 0 );
      
      // Add sort by dropdown label.
      var oSortByLabelSpan = $('<span/>', {
         'id': this.sSortByLabelID,
         'class': 'label sortByLabel'
      });
      oSortByLabelSpan.html(TranslateString("STR_SORT_BY"));
      oFilterMenuDiv.append(oSortByLabelSpan);
      
      // Add sort by dropdown.
      var oSortBySelect = $('<select/>', {
         'id': this.sSortBySelectID,
         'class': 'sortBySelect' + sPopupTabbableClass
      });
      var nNumSortLists = this.oUDFilter.numSortLists();
      for ( var lIndex = 0; lIndex < nNumSortLists; ++lIndex )
      {
         this.oUDFilter.SelectSortList( lIndex );
         var sortListName = this.oUDFilter.sortListName();
         var oOption = $('<option/>', {value: sortListName});
         oOption.html(TranslateString(sortListName));
         oSortBySelect.append(oOption);
      }
      if (nNumSortLists == 1)
      {
         // Hide Sort By dropdown if there is only one option.
         oSortByLabelSpan.hide();
         oSortBySelect.hide();

         //oSortBySelect.prop('disabled', 'disabled');
      }
      this.oUDFilter.SelectSortList( 0 );
      oFilterMenuDiv.append(oSortBySelect);
   },
   
   drawDevices: function() {
      var oFilterGroupsDiv = this.$('.filterGroups').empty();
      var oDevices = this.model.get('oDevices');
      var oAttributes = this.model.get('oAttributes');
      
      // TODO: Calculate current values in MControlDeviceSelectorGroup since we're passing attributes to that model.
      var oCurrentValues;
      if (IsDefined(oAttributes))
      {
         oCurrentValues = oAttributes.CURRENT_VALUE;
         if (IsDefined(oCurrentValues))
         {
            oCurrentValues = oCurrentValues.split('|');
         }
      }
      else
      {
         oCurrentValues = oAttributes; // TODO: Do we need this?
      }
      for ( var gvIndex = 0; gvIndex < this.oUDFilter.numGroupValues(); ++gvIndex )
      {
         this.oUDFilter.SelectGroupValue( gvIndex );
         this.oUDFilter.SelectSortList();
         var oSortList = this.oUDFilter.sortList();
         var sGroupName = this.oUDFilter.groupName();
         var sGroupValueName = this.oUDFilter.groupValueName();
         var sControlName = this.getName();
         var sControlLabel = this.model.get('sControlLabel');
         var sGroupID = 'filterGroup' + gvIndex;
         var sType = this.model.get('sType');
         var oMControlDeviceSelectorGroup = new MControlDeviceSelectorGroup({
            'oSortList':oSortList,
            'sControlLabel':sControlLabel,
            'sGroupName':sGroupName,
            'sGroupValueName':sGroupValueName,
            'sControlName':sControlName,
            'oDevices':oDevices,
            'oCurrentValues':oCurrentValues,
            'oAttributes':oAttributes,
            'sType':sType,
            'bPopupTabbable':this.model.get("bPopupTabbable")
         });
         var oVControlDeviceSelectorGroup = new VControlDeviceSelectorGroup({
            'id':sGroupID,
            'model':oMControlDeviceSelectorGroup
         });
         oFilterGroupsDiv.append(oVControlDeviceSelectorGroup.$el);
      }
   },
   
   drawUnfilteredDevices: function() {
      var oFilterGroupsDiv = this.$('.filterGroups').empty();
      var oDevices = this.model.get('oDevices');
      var oAttributes = this.model.get('oAttributes');
      var oCurrentValues = Array();
      if (IsDefined(oAttributes))
      {
         oCurrentValues = oAttributes.CURRENT_VALUE;
         if (IsDefined(oAttributes.CURRENT_VALUE))
         {
            oCurrentValues = oAttributes.CURRENT_VALUE;
         }
         else
         {
            oCurrentValues = oAttributes.DEFAULT_VALUE;
         }
		 if (IsDefined(oCurrentValues))
		 {
		    oCurrentValues = oCurrentValues.split('|');
		 }
      }      
      /*this.oUDFilter.SelectGroupValue( gvIndex );
      this.oUDFilter.SelectSortList();
      var oSortList = this.oUDFilter.sortList();
      var sGroupName = this.oUDFilter.groupName();
      var sGroupValueName = this.oUDFilter.groupValueName();*/
      var oSortList = [];
      for (i = 0; i != oDevices.length; ++i)
      {
         oSortList.push(i);
      }
      var sControlName = this.getName();
      var sGroupID = 'noneGroup';
      var sType = this.model.get('sType');
      var sControlLabel = this.model.get('sControlLabel');
      var oMControlDeviceSelectorGroup = new MControlDeviceSelectorGroup({
         'oSortList':oSortList,
         'sControlLabel':sControlLabel,
         'sGroupName':"STR_NONE",
         'sGroupValueName':"STR_ALL",
         'sControlName':sControlName,
         'oDevices':oDevices,
         'oCurrentValues':oCurrentValues,
         'oAttributes':oAttributes,
         'sType':sType
      });
      var oVControlDeviceSelectorGroup = new VControlDeviceSelectorGroup({
         'id':sGroupID,
         'model':oMControlDeviceSelectorGroup
      });
      oFilterGroupsDiv.append(oVControlDeviceSelectorGroup.$el);
   },
   
   onFilterValueChange: function(e) {
      this.updateFilter(e.currentTarget.selectedIndex);
      this.$('.filterGroupSortLists').trigger('change_' + this.model.get('sType'));
   },
   
   updateFilter: function(nSelectedIndex) {
      this.oUDFilter.SelectFilterValue( nSelectedIndex );
      // Get current values to hold on to
      var currentFilterValues = {
         'groupName': this.oUDFilter.groupName(),
         'sortListName': this.oUDFilter.sortListName()
      };
      // Update the Group by dropdown since filter has changed.
      var groupSelector = document.getElementById(this.sGroupBySelectID);
      groupSelector.options.length = 0;
      for ( var i = 0; i < this.oUDFilter.numGroups(); ++i )
      {
         this.oUDFilter.SelectGroup( i );
         var groupName = this.oUDFilter.groupName();
         var newopt = document.createElement("option");
         newopt.text = TranslateString(groupName);
         newopt.value = groupName;
         groupSelector.options.add(newopt);
      }
      groupSelector.selectedIndex = 0;
      this.updateGroup( groupSelector.selectedIndex, false, currentFilterValues );
   },

   onGroupChange: function (e) {
      this.updateGroup(e.currentTarget.selectedIndex, true);
      // This is necessary to prevent crashes when switching Group By to Enclosure in the Unassigned Drives container page.
      ClearDeviceSelection();
   },
   
   updateGroup: function(nSelectedIndex, saveCurrentValues, currentFilterValues) {
      if (IsDefined(currentFilterValues)) {
         if (this.oUDFilter.numGroups() > 0) {
            nSelectedIndex = this.oUDFilter.matchGroup(currentFilterValues.groupName);
            this.updateGroupMenu(currentFilterValues.groupName);
         }
      }
      this.oUDFilter.SelectGroup( nSelectedIndex );
      this.oUDFilter.SelectGroupValue( 0 );
      var listSelector = document.getElementById(this.sSortBySelectID);
      listSelector.options.length = 0;
      var nNumSortLists = this.oUDFilter.numSortLists();
      var sSortByLabelSelector = '#' + this.sSortByLabelID;
      // Only display Sort By dropdown if more than one option exists. 
      if (nNumSortLists > 1)
      {
         // Display the label.
         this.$(sSortByLabelSelector).show();

         // Display the dropdown.
         listSelector.style.display = "";
      }
      else
      {
         this.$(sSortByLabelSelector).hide();
         listSelector.style.display = "none";
      }
      if (nNumSortLists > 0)
      {
         for (var lIndex = 0; lIndex < nNumSortLists; lIndex++)
         {
            this.oUDFilter.SelectSortList(lIndex);
            var sortListName = this.oUDFilter.sortListName();
            var newopt = document.createElement("option");
            newopt.text = TranslateString(sortListName);
            newopt.value = sortListName;
            listSelector.options.add(newopt);
         }
         listSelector.selectedIndex = 0;
         this.updateSortList( listSelector.selectedIndex, saveCurrentValues, currentFilterValues );
      }
   },
   
   isValid: function() {
      var sErrorMessage = "";
      var oAttributes = this.model.get('oAttributes');
      var nCount = this.getSelectedCount();
      var sCardinality = oAttributes.CARDINALITY;
	   var nMax;
	   var nMin;
	   var nIncrement;
      if (sCardinality == "Single")
      {
         nMax = 1;
         nMin = 1;
         nIncrement = 1;
      }
      else
      {
         nMax = parseInt(oAttributes.MAXIMUM_SELECTION);
         nMin = parseInt(oAttributes.MINIMUM_SELECTION);
         nIncrement = parseInt(oAttributes.INCREMENT);
         if (isNaN(nIncrement))
         {
            nIncrement = 1;
         }
      }
      if (isNaN(nMax) && isNaN(nMin))
      {
         //sErrorMessage = TranslateString(this.model.get("sName")) + ": " + TranslateString("STR_CAPABILITY_MISMATCH");
      }
      else if ((nCount < nMin || nCount > nMax) || nCount%nIncrement != 0)
      {
         var sControlLabel = this.model.get('sControlLabel');
         var sMsg;
         //var sName = TranslateString(this.model.get("sName"));
         if (nMin == nMax)
         {
            if (sControlLabel == "STR_SEL_DEST_ARRAY")
            {
               sMsg = 'STR_EXACT_NUMBER_ARRAYS_ERROR';
            }
            else
            {
               sMsg = 'STR_EXACT_NUMBER_DRIVES_ERROR';
            }
            sErrorMessage = TranslateString(sControlLabel) + ": " + TranslateString(sMsg + "|" + nMin);
         }
         else if (nIncrement == 1)
         {
            sMsg = 'STR_NUMBER_DRIVES_ERROR_NO_INC';
            sErrorMessage = TranslateString(sControlLabel) + ": " + TranslateString(sMsg + "|" + nMin + "|" + nMax);
         }
         else
         {
            sMsg = 'STR_NUMBER_DRIVES_ERROR';
            sErrorMessage = TranslateString(sControlLabel) + ": " + TranslateString(sMsg + "|" + nMin + "|" + nMax + "|" + nIncrement);
         }
      }
      return sErrorMessage;
   },
   
   onSortByChange: function(e) {
      this.updateSortList(e.currentTarget.selectedIndex, true);
   },
   
   updateSortList: function(nSelectedIndex, saveCurrentValues, currentFilterValues) {
      if (IsDefined(currentFilterValues)) {
         if (this.oUDFilter.numSortLists() > 0) {
            nSelectedIndex = this.oUDFilter.matchSortList(currentFilterValues.sortListName);
            this.updateSortListMenu(currentFilterValues.sortListName);
         }
      }
      this.oUDFilter.SelectSortList(nSelectedIndex);
      var sSelected = "";
      // If we save current values (group or sort list change), update the current values
      // Otherwise, clear the current values
      if (saveCurrentValues) {
         var oSelected = this.getValueArray();
         for (var i=0; i<oSelected.length; ++i) {
            sSelected += oSelected[i];
            if (i != oSelected.length -1) {
               sSelected += "|";
            }
         }
      }
      var oAttributes = this.model.get('oAttributes');
      if (IsDefined(oAttributes)) {
         oAttributes.CURRENT_VALUE = sSelected;
      }
      else {
         oAttributes = {
            "CURRENT_VALUE": sSelected
         };
         this.model.set({
            'oAttributes': oAttributes
         });
      }
      this.drawDevices();
      this.triggerResize();
   },

   updateSortListMenu: function(sSelectedSortList) {
      var sSelector = '#' + this.sSortBySelectID;
      this.$(sSelector).val(sSelectedSortList);
   },

   updateGroupMenu: function(sSelectedGroup) {
      var sSelector = '#' + this.sGroupBySelectID;
      this.$(sSelector).val(sSelectedGroup);
   },
   
   resizeWidth: function() {
      AVControl.prototype.resizeWidth.call(this);
      var oSortList = this.$('.filterGroupSortLists');
      var nWidth = this.$('.filterGroups').innerWidth();
	   var nPaddingLeft = parseInt(oSortList.css('padding-left'));
	   var nPaddingRight = parseInt(oSortList.css('padding-right'));
	   var nBorderLeft = parseInt(oSortList.css('border-left-width'));
	   var nBorderRight = parseInt(oSortList.css('border-right-width'));
      var nSortListWidth = nWidth -
          nPaddingLeft -
          nPaddingRight -
          nBorderLeft -
          nBorderRight;
      oSortList.width(nSortListWidth);

      // Make sure all items in Device Selector have the same width.
      // Assuming that the item subtitles section has the greatest width.
      var oItemSubtitles = this.$('.itemSubtitle');
      var nMaxItemSubtitleWidth = 0;
      for (var i=0; i<oItemSubtitles.length; ++i)
      {
         var nWidth = $(oItemSubtitles[i]).width();
         if (nWidth > nMaxItemSubtitleWidth)
         {
            nMaxItemSubtitleWidth = nWidth;
         }
      }
      nMaxItemSubtitleWidth += 1; // Give extra padding. Required for IE to not break into 2 lines.
      for (var i=0; i<oItemSubtitles.length; ++i)
      {
         $(oItemSubtitles[i]).width(nMaxItemSubtitleWidth);
      }
   }
});