var VControlRangeInput = AVControl.extend({

   events: {
      "change input#rangeInputTextInput" : "rangeInputTextChanged",
      "keydown input#rangeInputTextInput" : "numbersOnly"
   },
   
   initialize: function() {
      AVControl.prototype.initialize.call(this);
   },
   
   render: function() {
      var oRangeInputDiv = $('<div/>', {'class': "radioWrapper"});
      $(this.el).append(oRangeInputDiv);
      this.drawRangeInput();
      return this;
   },
   
   getValue: function() {
      var s = '';
      s = this.$('input[id=rangeInputTextInput]').val();
      return s;
   },
   
   isValid: function() {
      var errorMessage = "";
      errorMessage = this.validateTextInput("rangeInputTextInput");
      if (errorMessage != "")
      {
         errorMessage = TranslateString(this.model.get("sName")) + ": " + TranslateString(errorMessage);
      }
      return errorMessage;
   },
   
   drawRangeInput: function() {
      var oRangeInputDiv = this.$('.radioWrapper');
      var oValues = this.model.get("oValues");
      var oAttributes = this.model.get("oAttributes");
      // Convert values to seconds
      var nMin = oAttributes.MINIMUM_VALUE;
      var nMax = oAttributes.MAXIMUM_VALUE;
      var nDefault = oAttributes.DEFAULT_VALUE;
      var nCurrent = oAttributes.CURRENT_VALUE;
      var sRangeLabel = oAttributes.RangeLabel;
      var sName = this.model.get("sName");
      // Create range text input area and its label
      var oRangeInputTextDiv = $('<div/>', {'class': "textInputWrapper"});
      var oRangeInputTextInput = $('<input/>',
      {
         'id': "rangeInputTextInput",
         'class': "textInput popupTabbable",
         'type': "text",
         'name': sName,
         'value': nCurrent,
         'width': "60px",
         'maxlength': "5"
      });
      var oRangeInputTextLabel = $('<label/>',
      {
         'for': "rangeInputTextInput",
         'class': "textInputLabel"
      });
      oRangeInputTextLabel.html(TranslateString(sRangeLabel + "|" + nMin + "|" + nMax));
      var oRangeInputTextInfoBox = $('<label/>',
      {
         'for': "rangeInputTextInput",
         'class': "infoBox"
      });
      oRangeInputTextInfoBox.css("display", "none");
      oRangeInputTextDiv.append(oRangeInputTextInput);
      oRangeInputTextDiv.append(oRangeInputTextLabel);
      oRangeInputTextDiv.append(oRangeInputTextInfoBox);
      oRangeInputDiv.append(oRangeInputTextDiv);
   },
   
   rangeInputTextChanged: function(e) {
      this.validateTextInput(e.currentTarget.id);
   },
   
   validateTextInput: function(sID) {
      var errorMessage = "";
      var value = parseInt(this.$('input[id=' + sID + ']').val());
      var oAttributes = this.model.get("oAttributes");
      var nMin = parseInt(oAttributes.MINIMUM_VALUE);
      var nMax = parseInt(oAttributes.MAXIMUM_VALUE);
      if (isNaN(value))
      {
         this.$('.infoBox').css("display", "inline");
         errorMessage = "STR_INPUT_IS_NAN";
         this.$('.infoBox').html(TranslateString(errorMessage));
      }
      else if (value > nMax || value < nMin)
      {
         this.$('.infoBox').css("display", "inline");
         errorMessage = "STR_OUT_OF_RANGE";
         this.$('.infoBox').html(TranslateString(errorMessage));
      }
      else
      {
         this.$('.infoBox').css("display", "none");
      }
      return errorMessage;
   },
   
   numbersOnly: function(e) {
      var key = e.which || e.keyCode;
      var isNumber = false;
      if ( !e.shiftKey && !e.altKey && !e.ctrlKey &&
         // numbers   
         key >= 48 && key <= 57 ||
         // Numeric keypad
         key >= 96 && key <= 105 ||
         // Backspace and Tab and Enter
         key == 8 || key == 9 || key == 13 ||
         // Home and End
         key == 35 || key == 36 ||
         // left and right arrows
         key == 37 || key == 39 ||
         // Del and Ins
         key == 46 || key == 45 )
      {
         isNumber = true;
      }
      return isNumber;
   }
});