var VControlSizeControl = AVControl.extend({

   events: {
      "change input#sizeControlDefaultInput" : "sizeControlChanged",
      "change input#sizeControlMaxInput" : "sizeControlChanged",
      "change input#sizeControlMBRInput" : "sizeControlChanged",
      "change input#sizeControlOtherInput" : "sizeControlChanged",
      "change select#sizeControlUnitDropdown" : "sizeControlUnitChanged",
      "change input#sizeControlTextInput" : "sizeControlTextChanged",
      "keydown input#sizeControlTextInput" : "numbersOnly"
   },
   
   initialize: function() {
      AVControl.prototype.initialize.call(this);
   },

   render: function() { "use strict";
      var oSizeControlDiv = $('<div/>', {'class': "radioWrapper"});
      $(this.el).append(oSizeControlDiv);
      this.drawSizeControl();
      return this;
   },

   ConvertMinSize: function( nValue, sUnitTo )
   {
      var dValue = fromBlocks(nValue, sUnitTo );
      var nPrecision = 0;
      if ( sUnitTo == "STR_MIB" )
      {
         nPrecision = 0;
      }
      else if ( sUnitTo == "STR_GIB" )
      {
        nPrecision = 3;
      }
      else if ( sUnitTo == "STR_TIB" )
      {
         nPrecision = 6;
      }
      var n = Math.pow(10, nPrecision);
      dValue = Math.floor( dValue * n )/n;
      var minUnitSize = Math.ceil( toBlocks( Math.pow( 10, -nPrecision ), sUnitTo ) );
      var testValue = Number(ConvertSize( dValue, sUnitTo, "STR_BLOCKS" ));
      while ( testValue < nValue )
      {
         testValue += minUnitSize;
      }
      return ConvertSize( testValue, "STR_BLOCKS", sUnitTo );
   },
   
   getValue: function() {
      var s = '';
      var checked = this.$('input[name=' + this.model.get("sName") + ']:checked');
      if (checked.attr('id') == "sizeControlMaxInput" ||
          checked.attr('id') == "sizeControlDefaultInput" ||
          checked.attr('id') == "sizeControlMBRInput")
      {
         s = checked.val();
      }
      else if (checked.attr('id') == "sizeControlOtherInput")
      {
         var sUnit = this.$('.dropdownSelect option:selected').val();
         var nValue = this.$('input[id=sizeControlTextInput]').val();
         s = ConvertSize(nValue, sUnit, "STR_BLOCKS");
      }
      return s;
   },
   
   isValid: function() {
      var errorMessage = "";
      var checked = this.$('input[name=' + this.model.get("sName") + ']:checked');
      if (checked.attr('id') == "sizeControlOtherInput")
      {
         errorMessage = this.validateTextInput("sizeControlTextInput");
      }
      if (errorMessage != "")
      {
         errorMessage = TranslateString(this.model.get("sName")) + ": " + TranslateString(errorMessage);
      }
      return errorMessage;
   },
   
   drawSizeControl: function() {
      var oSizeControlDiv = this.$('.radioWrapper');
      var oValues = this.model.get("oValues");
      var oAttributes = this.model.get("oAttributes");
      // Convert values to seconds
      var nMin = oAttributes.MINIMUM_VALUE;
      var nMax = oAttributes.MAXIMUM_VALUE;
      var sMax = ConvertSize(nMax, "STR_BLOCKS", "STR_MIB", 0) + " (" + GUIConvertSize(nMax) + ")";
      sMax = TranslateString("GUI_LOGICAL_DRIVE_SIZE_MAX_SIZE") + ": " + sMax;
      var nDefault = oAttributes.DEFAULT_VALUE;
      var sDefault = ConvertSize(nDefault, "STR_BLOCKS", "STR_MIB", 0) + " (" + GUIConvertSize(nDefault) + ")";
      sDefault = TranslateString("GUI_LOGICAL_DRIVE_SIZE_RECOMMENDED_SIZE") + ": " + sDefault;
      var nMaxMBR = ConvertSize(2, "STR_TIB", "STR_BLOCKS", 0);
      var sMaxMBR = ConvertSize(nMaxMBR, "STR_BLOCKS", "STR_MIB", 0) + " (" + GUIConvertSize(nMaxMBR) + ")";
      sMaxMBR = TranslateString("GUI_LOGICAL_DRIVE_SIZE_MAX_FOR_MBR") + ": " + sMaxMBR;
      // If set on High, textbox value should default
      var sName = this.model.get("sName");
      var nMaxChecked = true;
      if (nDefault != nMax)
      {
         var oSizeControlDefaultDiv = $('<div/>', {'class': "radioInputWrapper"});
         var oSizeControlDefaultInput = $('<input/>',
         {
            'id': "sizeControlDefaultInput",
            'class': "radioInput popupTabbable",
            'type': "radio",
            'name': sName,
            'value': nDefault,
            'checked': "checked"
         });
         var oSizeControlDefaultLabel= $('<label/>',
         {
            'for': "sizeControlDefaultInput",
            'class': "radioInputLabel"
         });
         oSizeControlDefaultLabel.html(sDefault);
         oSizeControlDefaultDiv.append(oSizeControlDefaultInput);
         oSizeControlDefaultDiv.append(oSizeControlDefaultLabel);
         oSizeControlDiv.append(oSizeControlDefaultDiv);
         nMaxChecked = false;
      }
      else if (nMax > nMaxMBR)
      {
         var oSizeControlMBRDiv = $('<div/>', {'class': "radioInputWrapper"});
         var oSizeControlMBRInput = $('<input/>',
         {
            'id': "sizeControlMBRInput",
            'class': "radioInput popupTabbable",
            'type': "radio",
            'name': sName,
            'value': nMaxMBR
         });
         var oSizeControlMBRLabel= $('<label/>',
         {
            'for': "sizeControlMBRInput",
            'class': "radioInputLabel"
         });
         oSizeControlMBRLabel.html(sMaxMBR);
         oSizeControlMBRDiv.append(oSizeControlMBRInput);
         oSizeControlMBRDiv.append(oSizeControlMBRLabel);
         oSizeControlDiv.append(oSizeControlMBRDiv);
      }
      // Create max radio button
      var oSizeControlMaxDiv = $('<div/>', {'class': "radioInputWrapper"});
      var oSizeControlMaxInput = $('<input/>',
      {
         'id': "sizeControlMaxInput",
         'class': "radioInput popupTabbable",
         'type': "radio",
         'name': sName,
         'value': nMax
      });
      var oSizeControlMaxLabel= $('<label/>',
      {
         'for': "sizeControlMaxInput",
         'class': "radioInputLabel"
      });
      oSizeControlMaxLabel.html(sMax);
      oSizeControlMaxDiv.append(oSizeControlMaxInput);
      oSizeControlMaxDiv.append(oSizeControlMaxLabel);
      oSizeControlDiv.append(oSizeControlMaxDiv);
      if (nMaxChecked == true)
      {
         oSizeControlMaxInput.prop("checked", true);
      }
      // Create other radio button
      var oSizeControlOtherDiv = $('<div/>', {'class': "radioInputWrapper"});
      var oSizeControlOtherInput = $('<input/>',
      {
         'id': "sizeControlOtherInput",
         'class': "radioInput popupTabbable",
         'type': "radio",
         'name': sName
      });
      var oSizeControlOtherLabel = $('<label/>',
      {
         'for': "sizeControlOtherInput",
         'class': "radioInputLabel"
      });
      oSizeControlOtherLabel.html(TranslateString("GUI_LOGICAL_DRIVE_SIZE_OTHER"));
      oSizeControlOtherDiv.append(oSizeControlOtherInput);
      oSizeControlOtherDiv.append(oSizeControlOtherLabel);
      oSizeControlDiv.append(oSizeControlOtherDiv);
      // Create idle text input area and its label
      var oSizeControlTextDiv = $('<div/>', {'class': "textInputWrapper"});
      oSizeControlTextDiv.css("display", "none");
      var oSizeControlTextInput = $('<input/>',
      {
         'id': "sizeControlTextInput",
         'class': "textInput popupTabbable",
         'type': "text",
         'name': sName,
         'width': "80px"
      });
      var oSizeControlUnitDropdown = $('<select/>', {
         'id': "sizeControlUnitDropdown",
         'class': "dropdownSelect"
      });
      var oSizeControlMB = $('<option/>', {
         'class': "dropdownOption",
         'value': "STR_MIB"
      });
      oSizeControlMB.html(TranslateString("STR_MIB|"));
      oSizeControlUnitDropdown.append(oSizeControlMB);
      var oSizeControlGB = $('<option/>', {
         'class': "dropdownOption",
         'value': "STR_GIB"
      });
      oSizeControlGB.prop('selected', true);
      oSizeControlGB.html(TranslateString("STR_GIB|"));
      oSizeControlUnitDropdown.append(oSizeControlGB);
      var oSizeControlTB = $('<option/>', {
         'class': "dropdownOption",
         'value': "STR_TIB"
      });
      
      oSizeControlTB.html(TranslateString("STR_TIB|"));
      oSizeControlUnitDropdown.append(oSizeControlTB);
      var oSizeControlTextLabel = $('<label/>',
      {
         'for': "sizeControlTextInput",
         'class': "textInputLabel"
      });
      var oSizeControlTextInfoBox = $('<label/>',
      {
         'for': "sizeControlTextInput",
         'class': "infoBox"
      });
      oSizeControlTextInfoBox.css("display", "none");
      oSizeControlTextDiv.append(oSizeControlTextInput);
      oSizeControlTextDiv.append(oSizeControlUnitDropdown);
      oSizeControlTextDiv.append(oSizeControlTextLabel);
      oSizeControlTextDiv.append(oSizeControlTextInfoBox);
      oSizeControlDiv.append(oSizeControlTextDiv);
      this.updateRange();
   },
   
   sizeControlChanged: function(e) {
      this.updateTextDisplay(e.currentTarget.id);
   },
   
   updateTextDisplay: function(sID) {
      // If "disabled" or "high" are selected, do not show the text input
      // If "idle" is selected, show the text input
      if (sID == "sizeControlOtherInput")
      {
         this.$('.infoBox').css("display", "none");
         this.$('.textInputWrapper').css("display", "block");
         this.updateRange();
      }
      else
      {
         this.$('.textInputWrapper').css("display", "none");
      }
      //this.triggerResize();
   },
   
   sizeControlUnitChanged: function(e) {
      this.updateRange();
      this.validateTextInput("sizeControlTextInput");
   },
   
   updateRange: function(sID) {
      var sUnit = this.$('.dropdownSelect option:selected').val();
      var oAttributes = this.model.get("oAttributes");
      var nMin = oAttributes.MINIMUM_VALUE;
      var nMax = oAttributes.MAXIMUM_VALUE;
      var sMin = this.ConvertMinSize(nMin, sUnit);
      var sMax = ConvertSize(nMax, "STR_BLOCKS", sUnit)
      var sLabel = TranslateString("GUI_LOGICAL_DRIVE_SIZE_OTHER_RANGE") + ": ";
      sLabel = sLabel + sMin +  " - " + sMax;
      this.$('.textInputLabel').html(sLabel);
   },
   
   sizeControlTextChanged: function(e) {
      this.validateTextInput(e.currentTarget.id);
   },
   
   validateTextInput: function(sID) {
      var errorMessage = "";
      var sValue = this.$('input[id=' + sID + ']').val();
      var oAttributes = this.model.get("oAttributes");
      var sUnit = this.$('.dropdownSelect option:selected').val();
      var nValue = parseInt(ConvertSize(sValue, sUnit, "STR_BLOCKS"));
      if (sValue == "" || isNaN(nValue))
      {
         errorMessage = "STR_INPUT_IS_NAN";
      }
      else if (nValue > parseInt(oAttributes.MAXIMUM_VALUE) || nValue < parseInt(oAttributes.MINIMUM_VALUE))
      {
         errorMessage = "STR_OUT_OF_RANGE";
      }
      else
      {
         this.$('.infoBox').css("display", "none");
      }
      if (errorMessage != "")
      {
         this.$('.infoBox').css("display", "inline");
         this.$('.infoBox').html(TranslateString(errorMessage));
      }
      return errorMessage;
   },
   
   numbersOnly: function(e) {
      var key = e.which || e.keyCode;
      var isNumber = false;
      if ( !e.shiftKey && !e.altKey && !e.ctrlKey &&
         // numbers   
         key >= 48 && key <= 57 ||
         // Numeric keypad
         key >= 96 && key <= 105 ||
         // Ctrl-a
         e.ctrlKey && key == 65 ||
         // comma, period and minus, . on keypad
         key == 190 || key == 188 || key == 109 || key == 110 ||
         // Backspace and Tab and Enter
         key == 8 || key == 9 || key == 13 ||
         // Home and End
         key == 35 || key == 36 ||
         // left and right arrows
         key == 37 || key == 39 ||
         // Del and Ins
         key == 46 || key == 45 )
      {
         isNumber = true;
      }
      return isNumber;
   }
});