var VControlSlider = AVControl.extend({

   events: {
      "change input#sliderTextInput" : "sliderTextChanged",
      "keydown input#sliderTextInput" : "numbersOnly"
   },
   
   initialize: function() {
      AVControl.prototype.initialize.call(this);
   },
   
   render: function() {
      var oSliderDiv = $('<div/>', {'class': "sliderWrapper"});
      $(this.el).append(oSliderDiv);
      this.drawSlider();
      return this;
   },
   
   getValue: function() {
      var s = '';
      s = this.$('input[name=' + this.model.get("sName") + ']').val();
      return s;
   },
   
   isValid: function() {
      var errorMessage = this.validateTextInput("sliderTextInput");
      if (errorMessage != "")
      {
         errorMessage = TranslateString(this.model.get("sName")) + ": " + TranslateString(errorMessage);
      }
      return errorMessage;
   },
   
   drawSlider: function() {
      var oSliderDiv = this.$('.sliderWrapper');
      var oValues = this.model.get("oValues");
      var sName = this.model.get("sName");
      var oAttributes = this.model.get("oAttributes");
      // Create the slider
      var nMin = parseInt(oAttributes.MINIMUM_VALUE);
      var nMax = parseInt(oAttributes.MAXIMUM_VALUE);
      var nIncrement = parseInt(oAttributes.INCREMENT);
      var nDefault = parseInt(oAttributes.DEFAULT_VALUE);
      var nCurrent = parseInt(oAttributes.CURRENT_VALUE);
      if (isNaN(nCurrent))
      {
         nCurrent = nDefault;
      }
      var oSliderInputDiv = $('<div/>', {'class': "sliderInputWrapper"});
      var oSliderInput = $('<div/>', {'class': "sliderInput", id: sName}).slider({
         range: "min",
         value: nCurrent,
			min: nMin,
			max: nMax,
			step: nIncrement,
         name: sName
      });
      // Append the slider
      oSliderInputDiv.append(oSliderInput);
      oSliderDiv.append(oSliderInputDiv);
      
      // Create the slider label
      /*var oSliderLabelDiv = $('<div/>', {'class': "sliderLabel"});
      var oSliderLabelLeft = $('<div/>', {'class': "sliderLabelLeft"});
      var oSliderLabelRight = $('<div/>', {'class': "sliderLabelRight"});
      oSliderLabelLeft.html(TranslateString("STR_SLIDER_READ"));
      oSliderLabelRight.html(TranslateString("STR_SLIDER_WRITE"));
      // Append the slider label
      oSliderLabelDiv.append(oSliderLabelLeft);
      oSliderLabelDiv.append(oSliderLabelRight);
      oSliderDiv.append(oSliderLabelDiv);*/
      
      // Create the textbox and values
      var oSliderTextDiv = $('<div/>', { 'class': "textInputWrapper" });
      var oSliderTextInput = $('<input/>',
      {
         'id': "sliderTextInput",
         'class': "textInput popupTabbable",
         'type': "text",
         'name': sName,
         'value': nCurrent,
         'width': "45px",
         'maxlength': "3"
      });
      var oSliderTextLabel = $('<label/>',
      {
         'for': "sliderTextInput",
         'class': "textInputLabel"
      });
      var oSliderTextInfoBox = $('<label/>',
      {
         'for': "sliderTextInput",
         'class': "infoBox"
      });
      oSliderTextInfoBox.css("display", "none");
      // Append the textbox and values

      //oSliderTextLabel.html(TranslateString("STR_READ_WRITE|" + nCurrent + "|" + (100-nCurrent)));


      var oReadColor = $('<div/>', { 'class': "color-box color-read" });
      var oWriteColor = $('<div/>', { 'class': "color-box color-write" });
      var oTextLabel = TranslateString("STR_PERCENT|" + nCurrent) + 
                       oReadColor.clone().wrap('<p>').parent().html() +
                       TranslateString("STR_READ") + " / " + 
                       TranslateString("STR_PERCENT|" + (100-nCurrent)) +
                       oWriteColor.clone().wrap('<p>').parent().html() +
                       TranslateString("STR_WRITE");

      //oSliderTextLabel.html(TranslateString("STR_PERCENT|" + nCurrent));
      oSliderTextLabel.html(oTextLabel);
      oSliderTextDiv.append(oSliderTextInput);
      oSliderTextDiv.append(oSliderTextLabel);
      oSliderTextDiv.append(oSliderTextInfoBox);
      oSliderDiv.append(oSliderTextDiv);
      
      // Bind the slider slide function
      oSliderDiv.bind("slide", function(e, ui) {
         oSliderTextInput.val(ui.value);
         oSliderTextLabel.html(TranslateString("STR_READ_WRITE|" + ui.value + "|" + (100-ui.value)));
      });
      // Bind the slider slidestop function
      oSliderDiv.bind("slidestop", function(e, ui) {
         $('.infoBox').css("display", "none");
      });
   },
   
   sliderTextChanged: function(e) {
      this.validateTextInput(e.currentTarget.id);
   },
   
   validateTextInput: function(sID, isValid) {
      var errorMessage = "";
      var value = this.$('input[id=' + sID + ']').val();
      value = parseInt(value, '10');
      var oAttributes = this.model.get("oAttributes");
      var nMin = parseInt(oAttributes.MINIMUM_VALUE);
      var nMax = parseInt(oAttributes.MAXIMUM_VALUE);
      if (isNaN(value))
      {
         this.updateTextInput(sID);
      }
      else if (value > nMax || value < nMin)
      {
         errorMessage = "STR_OUT_OF_RANGE";
      }
      else if (value%5 != 0)
      {
         errorMessage = "STR_NOT_MULTIPLE_OF_5";
      }
      else
      {
         this.$('.infoBox').css("display", "none");
         this.updateSliderAndLabel(sID);
      }
      if (errorMessage != "")
      {
         this.$('.infoBox').css("display", "inline");
         this.$('.infoBox').html(TranslateString(errorMessage));
      }
      return errorMessage;
   },
   
   updateSliderAndLabel: function(sID) {
      var value = this.$('input[id=' + sID + ']').val();
      this.$('.sliderInput').slider('value', value);
      this.$('.textInputLabel').html(TranslateString("STR_READ_WRITE|" + value + '|' + (100-value)));
   },

   updateTextInput: function(sID) {
      var value = this.$('.sliderInput').slider("value");
      this.$('input[id=' + sID + ']').val(value);
      this.$('.textInputLabel').html(TranslateString("STR_READ_WRITE|" + value + '|' + (100-value)));
   },
   
   numbersOnly: function(e) {
      var key = e.which || e.keyCode;
      var isNumber = false;
      if ( !e.shiftKey && !e.altKey && !e.ctrlKey &&
         // numbers   
         key >= 48 && key <= 57 ||
         // Numeric keypad
         key >= 96 && key <= 105 ||
         // Backspace and Tab and Enter
         key == 8 || key == 9 || key == 13 ||
         // Home and End
         key == 35 || key == 36 ||
         // left and right arrows
         key == 37 || key == 39 ||
         // Del and Ins
         key == 46 || key == 45 )
      {
         isNumber = true;
      }
      return isNumber;
   }
});