var VControlSurfaceAnalysis = AVControl.extend({

   events: {
      // Update the control when a radio button is selected.
      "change input#surfaceAnalysisDisableInput" : "surfaceAnalysisChanged",
      "change input#surfaceAnalysisHighInput" : "surfaceAnalysisChanged",
      "change input#surfaceAnalysisIdleInput" : "surfaceAnalysisChanged",
      //"keydown input#surfaceAnalysisTextInput" : "surfaceAnalysisTextChanged",
      "change input#surfaceAnalysisTextInput" : "surfaceAnalysisTextChanged",
      "keydown input#surfaceAnalysisTextInput" : "numbersOnly"
   },
   
   initialize: function() {
      AVControl.prototype.initialize.call(this);
   },
   
   render: function() {
      var oSurfaceAnalysisDiv = $('<div/>', {'class': "radioWrapper"});
      $(this.el).append(oSurfaceAnalysisDiv);
      this.drawSurfaceAnalysis();
      return this;
   },
   
   getValue: function() {
      var s = '';
      var checked = this.$('input[name=' + this.model.get("sName") + ']:checked');
      if (checked.attr('id') == "surfaceAnalysisDisableInput")
      {
         s = checked.val();
      }
      else if (checked.attr('id') == "surfaceAnalysisHighInput")
      {
         s = checked.val();
      }
      else if (checked.attr('id') == "surfaceAnalysisIdleInput")
      {
         s = this.$('input[id=surfaceAnalysisTextInput]').val() * 10;
      }
      return s;
   },
   
   isValid: function() {
      var errorMessage = "";
      var checked = this.$('input[name=' + this.model.get("sName") + ']:checked');
      if (checked.attr('id') == "surfaceAnalysisIdleInput") {
         errorMessage = this.validateTextInput("surfaceAnalysisTextInput");
      }
      if (errorMessage != "")
      {
         errorMessage = TranslateString(this.model.get("sName")) + ": " + TranslateString(errorMessage);
      }
      return errorMessage;
   },
   
   drawSurfaceAnalysis: function() {
      var oSurfaceAnalysisDiv = this.$('.radioWrapper');
      var oValues = this.model.get("oValues");
      var oAttributes = this.model.get("oAttributes");
      // Convert values to seconds
      var nMin = oAttributes.MINIMUM_VALUE / 10;
      var nMax = oAttributes.MAXIMUM_VALUE / 10;
      var nDefault = oAttributes.DEFAULT_VALUE / 10;
      // If set on High, textbox value should default
      var nCurrent = oAttributes.CURRENT_VALUE;
      if (nCurrent != 1)
      {
         nCurrent = nCurrent / 10;
      }
      else
      {
         nCurrent = nDefault;
      }
      var sName = this.model.get("sName");
      // Create disable radio button
      var oSurfaceAnalysisDisableDiv = $('<div/>', {'class': "radioInputWrapper"});
      var oSurfaceAnalysisDisableInput = $('<input/>',
      {
         'id': "surfaceAnalysisDisableInput",
         'class': "radioInput popupTabbable",
         'type': "radio",
         'name': sName,
         'value': "0"      
      });
      var oSurfaceAnalysisDisableLabel = $('<label/>',
      {
         'for': "surfaceAnalysisDisableInput",
         'class': "radioInputLabel"
      });
      oSurfaceAnalysisDisableLabel.html(TranslateString("STR_DISABLED"));
      oSurfaceAnalysisDisableDiv.append(oSurfaceAnalysisDisableInput);
      oSurfaceAnalysisDisableDiv.append(oSurfaceAnalysisDisableLabel);
      oSurfaceAnalysisDiv.append(oSurfaceAnalysisDisableDiv);
      // Create high radio button
      var oSurfaceAnalysisHighDiv = $('<div/>', {'class': "radioInputWrapper"});
      var oSurfaceAnalysisHighInput = $('<input/>',
      {
         'id': "surfaceAnalysisHighInput",
         'class': "radioInput popupTabbable",
         'type': "radio",
         'name': sName,
         'value': "1"      
      });
      var oSurfaceAnalysisHighLabel = $('<label/>',
      {
         'for': "surfaceAnalysisHighInput",
         'class': "radioInputLabel"
      });
      oSurfaceAnalysisHighLabel.html(TranslateString("ATTR_VALUE_SURFACE_ANALYSIS_MODE_AGGRESSIVE_TRUE"));
      oSurfaceAnalysisHighDiv.append(oSurfaceAnalysisHighInput);
      oSurfaceAnalysisHighDiv.append(oSurfaceAnalysisHighLabel);
      oSurfaceAnalysisDiv.append(oSurfaceAnalysisHighDiv);
      // Create idle radio button
      var oSurfaceAnalysisIdleDiv = $('<div/>', {'class': "radioInputWrapper"});
      var oSurfaceAnalysisIdleInput = $('<input/>',
      {
         'id': "surfaceAnalysisIdleInput",
         'class': "radioInput popupTabbable",
         'type': "radio",
         'name': sName
      });
      var oSurfaceAnalysisIdleLabel = $('<label/>',
      {
         'for': "surfaceAnalysisIdleInput",
         'class': "radioInputLabel"
      });
      oSurfaceAnalysisIdleLabel.html(TranslateString("ATTR_VALUE_SURFACE_ANALYSIS_MODE_AGGRESSIVE_FALSE"));
      oSurfaceAnalysisIdleDiv.append(oSurfaceAnalysisIdleInput);
      oSurfaceAnalysisIdleDiv.append(oSurfaceAnalysisIdleLabel);
      oSurfaceAnalysisDiv.append(oSurfaceAnalysisIdleDiv);
      // Create idle text input area and its label
      var oSurfaceAnalysisTextDiv = $('<div/>', {'class': "textInputWrapper"});
      oSurfaceAnalysisTextDiv.css("display", "none");
      var oSurfaceAnalysisTextInput = $('<input/>',
      {
         'id': "surfaceAnalysisTextInput",
         'class': "textInput popupTabbable",
         'type': "text",
         'name': sName,
         'value': nCurrent,
         'width': "40px",
         'maxlength': "2"
      });
      var oSurfaceAnalysisTextLabel = $('<label/>',
      {
         'for': "surfaceAnalysisTextInput",
         'class': "textInputLabel"
      });
      oSurfaceAnalysisTextLabel.html(TranslateString("STR_NOTE_SURFACE_SCAN_DELAY|" + nMin + "|" + nMax));
      var oSurfaceAnalysisTextInfoBox = $('<label/>',
      {
         'for': "surfaceAnalysisTextInput",
         'class': "infoBox"
      });
      oSurfaceAnalysisTextInfoBox.css("display", "none");
      oSurfaceAnalysisTextDiv.append(oSurfaceAnalysisTextInput);
      oSurfaceAnalysisTextDiv.append(oSurfaceAnalysisTextLabel);
      oSurfaceAnalysisTextDiv.append(oSurfaceAnalysisTextInfoBox);
      oSurfaceAnalysisDiv.append(oSurfaceAnalysisTextDiv);
      // Check the current value and select the appropriate radio
      if (oAttributes.CURRENT_VALUE == 0)
      {
         oSurfaceAnalysisDisableInput.prop("checked", true);
      }
      else if (oAttributes.CURRENT_VALUE == 1)
      {
         oSurfaceAnalysisHighInput.prop("checked", true);
      }
      else
      {
         oSurfaceAnalysisIdleInput.prop("checked", true);
         oSurfaceAnalysisTextDiv.css("display", "block");
      }
   },
   
   surfaceAnalysisChanged: function(e) {
      this.updateTextDisplay(e.currentTarget.id);
   },
   
   surfaceAnalysisTextChanged: function(e) {
      this.validateTextInput(e.currentTarget.id);
   },
   
   updateTextDisplay: function(sID) {
      // If "disabled" or "high" are selected, do not show the text input
      // If "idle" is selected, show the text input
      if (sID == "surfaceAnalysisIdleInput")
      {
         this.$('.infoBox').css("display", "none");
         this.$('.textInputWrapper').css("display", "block");
      }
      else
      {
         this.$('.textInputWrapper').css("display", "none");
      }
      this.triggerResize();
   },
   
   validateTextInput: function(sID) {
      var errorMessage ="";
      var value = parseInt(this.$('input[id=' + sID + ']').val());
      var oAttributes = this.model.get("oAttributes");
      var nMin = parseInt(oAttributes.MINIMUM_VALUE) / 10;
      var nMax = parseInt(oAttributes.MAXIMUM_VALUE) / 10;
      if (isNaN(value))
      {
         this.$('.infoBox').css("display", "inline");
         errorMessage = "STR_INPUT_IS_NAN";
         this.$('.infoBox').html(TranslateString(errorMessage));
      }
      else if (value > nMax || value < nMin)
      {
         errorMessage = "STR_OUT_OF_RANGE";
         this.$('.infoBox').css("display", "inline");
         this.$('.infoBox').html(TranslateString(errorMessage));
      }
      else
      {
         this.$('.infoBox').css("display", "none");
      }
      return errorMessage;
   },
   
   numbersOnly: function(e) {
      var key = e.which || e.keyCode;
      var isNumber = false;
      if ( !e.shiftKey && !e.altKey && !e.ctrlKey &&
         // numbers   
         key >= 48 && key <= 57 ||
         // Numeric keypad
         key >= 96 && key <= 105 ||
         // Ctrl-a
         e.ctrlKey && key == 65 ||
         // Backspace and Tab and Enter
         key == 8 || key == 9 || key == 13 ||
         // Home and End
         key == 35 || key == 36 ||
         // left and right arrows
         key == 37 || key == 39 ||
         // Del and Ins
         key == 46 || key == 45 )
      {
         isNumber = true;
      }
      return isNumber;
   }
});