var VDeviceLabel = BaseView.extend({
   
   initialize: function() {
      BaseView.prototype.initialize.call(this);
      this.render();
   },
   
   render: function() {
      this.drawLabel();
      return this;
   },
   
   drawLabel: function() {
      var sDeviceID = this.model.get("sDeviceID");
      var sIconClass = this.model.get('sIconClass');
      if (IsDefinedAndSet(sIconClass))
      {
         var oIconDiv = $('<div/>', {'class': 'icon HPSSA_SPRITE ' + sIconClass});
         this.$el.append(oIconDiv);
         // Add Status Icon Overlay if device does not have OK status.
         var oIconOverlayDiv = $('<div/>', {'class': 'iconOverlay'});;
         var sIconStatusOverlayClass = this.model.get('sIconStatusOverlayClass');
         if (IsDefinedAndSet(sIconStatusOverlayClass) && sIconStatusOverlayClass != "ICON_OVERLAY_STATUS_OK")
         {
            oIconOverlayDiv.addClass('HPSSA_SPRITE');
            oIconOverlayDiv.addClass(sIconStatusOverlayClass);
         }
         else {
            // Drive Transient type can be considered a "status".
            // TODO: Change this in lower layers.
            var sIconTypeOverlayClass = this.model.get('sIconTypeOverlayClass');
            if (IsDefinedAndSet(sIconTypeOverlayClass) && sIconTypeOverlayClass == "ICON_OVERLAY_TYPE_TRANSIENT")
            {
               oIconOverlayDiv.addClass('HPSSA_SPRITE');
               oIconOverlayDiv.addClass(sIconTypeOverlayClass);
            }
         }
         // Add empty div as placeholder in case a state icon overlay exists.
         oIconDiv.append(oIconOverlayDiv);

         // Add State Icon Overlay (i.e. encrypted device) if one exists
         var sIconStateOverlayClass = this.model.get('sIconStateOverlayClass');
         if (IsDefinedAndSet(sIconStateOverlayClass))
         {
            var oIconStateOverlayDiv = $('<div/>', {'class': 'iconStateOverlay HPSSA_SPRITE ' + sIconStateOverlayClass});
            oIconDiv.append(oIconStateOverlayDiv);
         }
         // Uncomment the following section to test the State Overlay icon.
         /*else
         {
            var oIconStateOverlayDiv = $('<div/>', {'class': 'iconStateOverlay HPSSA_SPRITE ICON_OVERLAY_STATE_ENCRYPTION'});
            oIconDiv.append(oIconStateOverlayDiv);
         }*/

         // Inactive Task overlay and Active Icon overlay are mutually exclusive.

         // Add Inactive Task Icon Overlay if one exists.
         var sIconInactiveTaskClass = this.model.get('sIconInactiveTaskClass');
         if (IsDefinedAndSet(sIconInactiveTaskClass) && sIconInactiveTaskClass != "ICON_OVERLAY_ACTIVITY_NONE")
         {
            var oIconInactiveTaskOverlayDiv = $('<div/>', {'class': 'iconInactiveTaskOverlay HPSSA_SPRITE ' + sIconInactiveTaskClass});
            oIconDiv.append(oIconInactiveTaskOverlayDiv);
         }
         // Add Active Icon Overlay if device has Active status.
         var sIconActivityClass = this.model.get('sIconActivityClass');
         if (IsDefinedAndSet(sIconActivityClass) && sIconActivityClass != "ICON_OVERLAY_ACTIVITY_NONE")
         {
            var oIconActivityOverlayDiv = $('<div/>', {'class': 'iconActivityOverlay HPSSA_SPRITE ' + sIconActivityClass});
            oIconDiv.append(oIconActivityOverlayDiv);
         }
         /*var sIconTypeOverlayClass = this.model.get('sIconTypeOverlayClass');
         if (IsDefinedAndSet(sIconTypeOverlayClass))
         {
            // TODO: Show Assigned Spare icon. Don't have an icon for it right now.
         }*/
      }
      
      var oLabelDiv = $('<div/>', {'class': 'label'});
      oLabelDiv.append(this.model.get('sLabel'));
      
      var sDescription = this.model.get('sDescription');
      var oPluralDiv = $('<div/>', {'class': 'plural'});
      this.$el.append(oPluralDiv);
      if (IsDefinedAndSet(sDescription))
      {
         oPluralDiv.append(oLabelDiv);
         var oDescriptionDiv = $('<div/>', {'class': 'description'});
         oDescriptionDiv.append(sDescription);
         // Add links are enabled, add them to the right of the description.
         var sLink = this.model.get('sLink');
         if (IsDefinedAndSet(sLink))
         {
            oDescriptionDiv.append(" | ");
            var oDescLink = $('<a/>', {
               'href': '#',
               'id': sLink,
               'class': 'Action'
            });
            oDescLink.append(TranslateString(sLink));
            oDescriptionDiv.append(oDescLink);
         }

         oPluralDiv.append(oDescriptionDiv);
      }
      else
      {
         var oSingleDiv = $('<div/>', {'class': 'single'});
         oSingleDiv.append(oLabelDiv);
         oPluralDiv.append(oSingleDiv);
      }
      // Append wear gauge bar
      var nSSDWear = this.model.get('nSSDWear');
      var nSSDStatus = this.model.get('nSSDStatus');
      if (IsDefinedAndSet(nSSDWear))
      {
         this.drawWearGaugeBar(nSSDWear, nSSDStatus);
      }
      this.$el.append(GetClearFloatsDiv());
   },

   drawWearGaugeBar: function(nSSDWear, nSSDStatus) {
      var oWearGaugeWrapper = $('<div/>', {'class': 'wearGaugeWrapper'});

      // Append text information (i.e. XX%)
      var oLabelDiv = $('<div/>', {'class': 'label'});
      // Make sure wear is 2 decimals
      nSSDWear = Math.round(nSSDWear*100)/100;
      oLabelDiv.append(TranslateString("STR_PERCENT|" + nSSDWear));
      oWearGaugeWrapper.append(oLabelDiv);

      // Append graphical wear gauge
      var oWearGaugeBarDiv = $('<div/>', {'class': 'wearGaugeBarWrapper'});
      // Determine color/percentage for wear gauge
      var sWearGaugeStatus = "wearGaugeOK";
      if (nSSDStatus == 1)
      {
         sWearGaugeStatus = "wearGaugeFailed";
      }
      else if (nSSDStatus == 2)
      {
         sWearGaugeStatus = "wearGaugeTripped";
      }
      else if (nSSDStatus == 3)
      {
         sWearGaugeStatus = "wearGaugeInfo";
      }
      // Format the wear percentage
      if (nSSDWear < 2)
      {
         nSSDWear = 2; // Display 2% for anything less than 2%
      }
      var sGaugeDisplayFull = nSSDWear + "%";
      var sGaugeDisplayEmpty = 100 - nSSDWear + "%";
      // Build the bar
      var oWearGaugeBar = $('<table/>', {
         'class':'wearGaugeBarTable'
      });
      var oWearGaugeBarTR = $('<tr/>', {
         'class':'wearGaugeBarTR'
      });
      var oWearGaugeBarDone = $('<td/>', {
         'id': 'wearGaugeFull',
         'class':sWearGaugeStatus,
         'width':sGaugeDisplayFull
      });
      var oWearGaugeBarNotDone = $('<td/>', {
         'id': 'wearGaugeEmpty',
         'class':'wearGaugeEmpty',
         'width':sGaugeDisplayEmpty
      });
      oWearGaugeBarTR.append(oWearGaugeBarDone);
      oWearGaugeBarTR.append(oWearGaugeBarNotDone);
      oWearGaugeBar.append(oWearGaugeBarTR);
      oWearGaugeBarDiv.append(oWearGaugeBar);
      oWearGaugeWrapper.append(oWearGaugeBarDiv);

      this.$el.append(oWearGaugeWrapper);
   }
   
});