var VPopup = AVResize.extend({
   
   className: "popupDiv",

   initialize: function() {
      AVResize.prototype.initialize.call(this);
      this.oVPopupForeground = null;
      this.oVPopupBackground = null; // Background block overlay.
      this.$el.addClass(this.model.get('sType'));
      this.render();
   },

   dispose: function() {
      this.disposeForeground();
      this.disposeBackground();
      AVResize.prototype.dispose.call(this);
   },

   disposeForeground: function() {
      if (IsDefined(this.oVPopupForeground))
      {
         this.oVPopupForeground.dispose();
         this.oVPopupForeground = null;
      }
   },

   disposeBackground: function() {
      if (IsDefined(this.oVPopupBackground))
      {
         this.oVPopupBackground.dispose();
         this.oVPopupBackground = null;
      }
   },

   onDefault: function() {
      this.oVPopupForeground.onDefault();
   },

   onDismiss: function() {
      this.oVPopupBackground.onDismiss();
   },
   
   setFocus: function() {
      this.model.get('oVContent').setFocus();
   },

   render: function() {
      // Draw Popup.
      var oM = this.model;
      var bUseWindowWidth = oM.get('bUseWindowWidth');
      if (!IsDefined(bUseWindowWidth))
      {
         bUseWindowWidth = false;
      }
      var oMPopupForeground = new MPopupForeground({
         'sAlign': oM.get('sAlign'),
         'nTop': oM.get('nTop'),
         'oVContent': oM.get('oVContent'),
         'nBorderRadius': oM.get('nBorderRadius'),
         'sBackgroundColor': oM.get('sBackgroundColor'),
         'sPadding': oM.get('sPadding'),
         'nPopupWidth': oM.get('nPopupWidth'),
         'bUseWindowWidth': bUseWindowWidth,
         'sBorderWeight': oM.get('sBorderWeight'),
         'sBorderColor': oM.get('sBorderColor')
      });
      this.oVPopupForeground = new VPopupForeground({
         'model': oMPopupForeground,
      });
      this.$el.append(this.oVPopupForeground.$el);

      // Draw page background block overlay.
      var oMPopupBackground = new MPopupBackground({
         'sDisableColor': oM.get('sDisableColor'),
         'nDisableOpacity': oM.get('nDisableOpacity'),
         'bClickOutsideToClose': oM.get('bClickOutsideToClose')
      });
      this.oVPopupBackground = new VPopupBackground({
         'model': oMPopupBackground,
      });
      this.$el.append(this.oVPopupBackground.$el);

      return this;
   },

   resizeWidth: function() {
      AVResize.prototype.resizeWidth.call(this);
      var nWidth = this.model.get("nWidth");
      this.oVPopupForeground.model.set("nWidth", nWidth);
      this.oVPopupBackground.model.set("nWidth", nWidth);
   },
   
   resizeHeight: function() {
      AVResize.prototype.resizeHeight.call(this);
      var nHeight = this.model.get("nHeight");
      this.oVPopupForeground.model.set("nMaxHeight", nHeight);
      this.oVPopupForeground.resizeHeight();
      this.oVPopupBackground.model.set("nHeight", nHeight);
   }
});

// Popup Foreground

var MPopupForeground = AMResize.extend({
});

var VPopupForeground = AVResize.extend({
   
   className: "outerModalPopupDiv",

   initialize: function() {
      AVResize.prototype.initialize.call(this);
      this.render();
   },

   dispose: function() {
      this.disposeContent();
      AVResize.prototype.dispose.call(this);
   },

   disposeContent: function() {
      var oVContent = this.model.get('oVContent');
      if (IsDefined(oVContent))
      {
         oVContent.dispose();
      }
   },
   
   render: function() {
      var oInnerDiv = $('<div/>', {'class': 'innerModalPopupDiv'});
      var oVContent = this.model.get('oVContent');
      if (IsDefined(oVContent))
      {
         oInnerDiv.append(oVContent.$el);
      }
      this.$el.append(oInnerDiv);

      var sPopupDisplay = 'block';
      var sPopupWidth = this.model.get('nPopupWidth') + 'px';
      var sPopupBorderRadius = this.model.get('nBorderRadius') + 'px';
      var sPopupPosition = 'absolute';
      var nPopupZIndex = 100;
      var sInnerBorderRadius = (this.model.get('nBorderRadius') - 3) + 'px';
      
      // TODO: For menus - implement later.
      /*oPopupDiv.css({
         'borderColor': '#0096D6',
         'borderStyle': 'solid'
      })*/

      this.$el.css({
         'display': sPopupDisplay,
         'top': this.model.get('nTop'),
         'width': sPopupWidth,
         'padding': this.model.get('sBorderWeight'),
         'background': this.model.get('sBorderColor'),
         'borderRadius': sPopupBorderRadius,
         'MozBorderRadius': sPopupBorderRadius,
         'WebkitBorderRadius': sPopupBorderRadius,
         'position': sPopupPosition,
         'zIndex': nPopupZIndex,
         'min-height': this.model.get('sMinHeight')
      });

      oInnerDiv.css({
         'padding': this.model.get('sPadding'),
         'background': this.model.get('sBackgroundColor'),
         'borderRadius': sInnerBorderRadius,
         'MozBorderRadius': sInnerBorderRadius,
         'WebkitBorderRadius': sInnerBorderRadius
      });
      
      return this;
   },

   // Called when ENTER keyboard control is pressed.
   onDefault: function() {
      var oVContent = this.model.get('oVContent');
      if (IsDefined(oVContent))
      {
         oVContent.onDefault();
      }
   },

   resizeWidth: function() {
      // Take the width and calculate margins to help center the div.
      var oM = this.model;
      var nWindowWidth = oM.get('nWidth');
      var bUseWindowWidth = oM.get('bUseWindowWidth');
      var nPopupWidth;
      if (bUseWindowWidth)
      {
         nPopupWidth = nWindowWidth;
         this.$el.width(nPopupWidth);
      }
      else
      {
         nPopupWidth = oM.get('nPopupWidth');
         var nLeft = 0; // Default to align = left
         if (nWindowWidth > nPopupWidth && oM.get('sAlign') == 'center')
         {
            nLeft = (nWindowWidth - nPopupWidth)/2;
         }
         this.$el.css('left', nLeft);
      }
      var oVContent = oM.get('oVContent');
      oVContent.model.set('nWidth', nPopupWidth);
      //oVContent.resizeWidth();
   },

   resizeHeight: function() {
      var oVContent = this.model.get('oVContent');
      var nMaxHeight = this.model.get('nMaxHeight');
      var nPopupMaxHeight = nMaxHeight - 22; // 10 padding top, 10 padding bottom, 2 for border
      oVContent.model.set('nMaxHeight', nPopupMaxHeight);
      oVContent.resizeHeight();
   }
});

// Popup Background

var MPopupBackground = AMResize.extend({
   defaults: {
      'bClickOutsideToClose':false
   }
});

var VPopupBackground = AVResize.extend({
   
   className: "blockModalPopupDiv",

   events: {
      'click': 'onDismiss'
   },

   initialize: function() {
      AVResize.prototype.initialize.call(this);
      this.render();
   },

   dispose: function() {
      this.$el.empty();
      AVResize.prototype.dispose.call(this);
   },

   onDismiss: function() {
      if (this.model.get('bClickOutsideToClose'))
      {
         this.$el.trigger("closePopup");
      }
      return false;
   },

   render: function() {
      // Set styles for the page block overlay.
      var sBlockWidth = 100 + '%';
      var sBlockBorder = 0 + 'px';
      var sBlockPadding = 0 + 'px';
      var sBlockMargin = 0 + 'px';
      var sBlockBackground = this.model.get('sDisableColor');
      var sBlockOpacity = (this.model.get("nDisableOpacity") / 100);
      var sBlockFilter = 'alpha(Opacity=' + this.model.get("nDisableOpacity") + ')';
      var nBlockZIndex = 99;
      var sBlockPosition = "fixed";
      var sBlockTop = 0 + 'px';
      var sBlockLeft = 0 + 'px';
      var nHeight = this.model.get('nHeight');

      this.$el.css({
         'width': sBlockWidth,
         'border': sBlockBorder,
         'padding': sBlockPadding,
         'margin': sBlockMargin,
         'background': sBlockBackground,
         'opacity': sBlockOpacity,
         'filter': sBlockFilter,
         'zIndex': nBlockZIndex,
         'position': sBlockPosition,
         'top': sBlockTop,
         'left': sBlockLeft,
         'height': nHeight
      });
      return this;
   },

   resizeWidth: function() {
      AVResize.prototype.resizeWidth.call(this);
   },
   
   resizeHeight: function() {
      AVResize.prototype.resizeHeight.call(this);
   }

});

var MDetailsPopup = AMPopupContent.extend({
   defaults: {
      'bUseMaxHeight': 'true'
   }
});

var VDetailsPopup = AVPopupContent.extend({

   events: {
      'click .navForward':'clickNavButton',
      'click .navBackward':'clickNavButton',
      'click .button.pianoButton.Dismiss.op': 'onDismiss'
   },
   
   initialize: function() {
      AVResize.prototype.initialize.call(this);
      this.oVPopupBody = null;
      this.render();
   },

   render: function() {
      var sDeviceID = this.model.get('sDeviceID');
      this.model.set({'sInitialDeviceID':sDeviceID});
      var oDevice = this.model.get('oDevice');

      // Create an object to store history
      this.oHistory = [];
      this.oHistory.push(sDeviceID);
      this.nHistoryPage = 0;

      if (!IsDefined(oDevice))
      {
         oDevice = GetDeviceDetails(sDeviceID, 3).device;
      }
      var bIncludeStatusAlerts = this.model.get('bIncludeStatusAlerts');

      var oMDeviceDetails = new MDeviceDetails({
         'sDeviceID': sDeviceID,
         'oDevice': oDevice,
         'bIncludeStatusAlerts': bIncludeStatusAlerts,
         'bIncludeDeviceLinks': true
      });
      this.oVPopupBody = new VDeviceDetails({
         'className': 'popupDetails',
         'model': oMDeviceDetails
      });
      oMDeviceDetails.bind("change:sDeviceID", this.clickedDeviceLink, this);

      // Header
      var oHeader = this.drawHeader();
      var oNavDiv = this.drawHeaderNav(oHeader);
      // Header Right - Status Counts
      this.updateDeviceStatusCounts(oDevice);

      // Body
      this.$el.append(this.oVPopupBody.$el);

      // Footer
      this.drawFooter();

      return this;
   },

   updateDeviceStatusCounts: function(oDevice) {
      var bIncludeStatusAlerts = this.model.get('bIncludeStatusAlerts');
      var oHeader = this.$('.popupHeader');
      if (bIncludeStatusAlerts)
      {
         if (IsDefined(this.oVDeviceStatusCounts))
         {
            this.oVDeviceStatusCounts.dispose();
            this.$('headerStatusCounts').remove();
         }
         // Add Status Count Summary (w/ icons) to the header.
         var oMDeviceStatusCounts = new MDeviceStatusCounts({
            'oDevice': oDevice
         });
         this.oVDeviceStatusCounts = new VDeviceStatusCounts({
            'className':'headerStatusCounts',
            'model': oMDeviceStatusCounts
         });
         oHeader.append(this.oVDeviceStatusCounts.$el);
         this.oVPopupBody.listenTo(this.oVDeviceStatusCounts, 'scrollToStatus', this.oVPopupBody.scrollToStatus);
      }
   },

   dispose: function() {
      this.oHistory = null;
      this.nHistoryPage = null;
      this.$el.empty();
      AVPopupContent.prototype.dispose.call(this);
   },

   clickedDeviceLink: function() {
      this.updateDetails(true);
      var sDeviceID = this.model.get('sDeviceID');
      this.updateHistory(sDeviceID);
   },

   updateDetails: function(isDeviceLink) {
      // Update information display from this model (we came clickNavButton)
      var sNewDeviceID = this.model.get('sDeviceID');
      var oNewDevice = this.model.get('oDevice');

      // Update information based on model (we came from clickedDeviceLink)
      if (isDeviceLink) {
         sNewDeviceID = this.oVPopupBody.model.get('sDeviceID');
         oNewDevice = this.oVPopupBody.model.get('oDevice');
      }

      // Update header
      this.$('.deviceLabel').remove();
      var oDeviceLabelDiv = $('<div/>', {'class': 'deviceLabel'});
      var oMDeviceLabel = new MDeviceLabel({
         'sDeviceID': sNewDeviceID
      });
      this.oVDeviceLabel = new VDeviceLabel({
         'model': oMDeviceLabel,
         'el': oDeviceLabelDiv
      });
      this.$('.popupHeader').prepend(this.oVDeviceLabel.$el);

      // Update popupDetails (Main Body)
      var oOldDiv = this.$('.popupDetails');
      oOldDiv.remove();
      var bIncludeStatusAlerts = this.model.get('bIncludeStatusAlerts');
      this.model.set({
         'sDeviceID': sNewDeviceID,
         'oDevice': oNewDevice
      });
      var oMDeviceDetails = new MDeviceDetails({
         'sDeviceID': sNewDeviceID,
         'oDevice': oNewDevice,
         'bIncludeStatusAlerts': bIncludeStatusAlerts,
         'bIncludeDeviceLinks': true
      });
      this.oVPopupBody = new VDeviceDetails({
         'className': 'popupDetails',
         'model': oMDeviceDetails
      });
      oMDeviceDetails.bind("change:sDeviceID", this.clickedDeviceLink, this);
      //this.oVPopupBody.listenTo(this.oVDeviceStatusCounts, 'scrollToStatus', this.oVPopupBody.scrollToStatus);
      this.updateDeviceStatusCounts(oNewDevice);
      this.$('.popupHeader').after(this.oVPopupBody.$el);
      this.resize();
   },

   clickNavButton: function(e) {
      var sClickedID = e.currentTarget.id;
      var bChangedPage = false;
      if (sClickedID == "navBackward") {
         if (this.nHistoryPage < this.oHistory.length-1) {
            ++this.nHistoryPage;
            bChangedPage = true;
         }
      }
      else if (sClickedID == "navForward") {
         if (this.nHistoryPage > 0) {
            --this.nHistoryPage;
            bChangedPage = true;
         }
      }
      if (bChangedPage) {
         var sNewDeviceID = this.oHistory[this.nHistoryPage];
         var oNewDevice = GetDeviceDetails(sNewDeviceID).device;
         this.model.set({
            'sDeviceID':sNewDeviceID,
            'oDevice':oNewDevice
         });
         this.updateDetails();
      }
      this.updateNavButtons();

      // Update the status alert summary in header.

      return false;
   },

   updateHistory: function(sNewDeviceID) {
      this.oHistory.splice(0, this.nHistoryPage);
      this.nHistoryPage = 0;
      this.oHistory.unshift(sNewDeviceID);
      this.updateNavButtons();
   },

   updateNavButtons: function() {
      var oNavBackwardButton = this.$('.navBackward');
      var oNavForwardButton = this.$('.navForward');
      var nHistoryPage = this.nHistoryPage;
      var oHistory = this.oHistory;
      if (oHistory.length == 1) {
         this.toggleNavButton(oNavBackwardButton, false)
         this.toggleNavButton(oNavForwardButton, false)
      }
      else {
         this.toggleNavButton(oNavBackwardButton, true)
         this.toggleNavButton(oNavForwardButton, true)
         if (nHistoryPage == 0) {
            this.toggleNavButton(oNavForwardButton, false)
         }
         else if (nHistoryPage == oHistory.length - 1) {
            this.toggleNavButton(oNavBackwardButton, false)
         }
      }
   },

   toggleNavButton: function(oNavButton, isEnabled) {
      if (isEnabled) {
         oNavButton.addClass('enabled');
         oNavButton.removeClass('disabled');
      }
      else {
         oNavButton.removeClass('enabled');
         oNavButton.addClass('disabled');
      }
   },

   onDismiss: function() {
      var sDeviceID = this.model.get('sInitialDeviceID');
      SelectDevice(sDeviceID);
      AVPopupContent.prototype.onDismiss.call(this);
      return false;
   }

});

var MStatusPopup = AMPopupContent.extend({
   defaults: {
      'bUseMaxHeight': true
   }
});

var VStatusPopup = AVPopupContent.extend({
   
   render: function() {
      var sDeviceID = this.model.get('sDeviceID');
      var oDevice = this.model.get('oDevice');

      this.drawHeader();

      // Body
      var oMDeviceStatus = new MDeviceStatus({
         'oDevice':oDevice,
         'sDeviceID': sDeviceID,
         'nDetailLevel':2
      });
      this.oVPopupBody = new VDeviceStatus({
         'className':'popupDetails',
         'model':oMDeviceStatus
      });
      this.$el.append(this.oVPopupBody.$el);
      
      // Footer
      this.drawFooter();
      
      return this;
   }

});

var MAlert = AMResize.extend({});

var VAlert = AVResize.extend({

   initialize: function() {
      AVResize.prototype.initialize.call(this);
      this.render();
   },
   
   render: function() {
      // Add body top padding.
      var oBodyPaddingDiv = $('<div/>', {'class': 'bodyPadding'});
      this.$el.append(oBodyPaddingDiv);

      // Add notes container.
      var oNotesDiv = $('<div/>', {'class': "notes"});
      this.$el.append(oNotesDiv);

      var oNotes = [{
         "NoteType": "CONFIRM",
         "NoteContent": [this.model.get('sMessage')]
      }];

      var oMCollapsedNote = new MCollapsedNote({
         'nDisplay':"0",
         'oNotes':oNotes
      });
      this.oVCollapsedNote = new VCollapsedNote({
         'model':oMCollapsedNote
      });
      oNotesDiv.append(this.oVCollapsedNote.$el);
   }

});

var MAlertPopup = AMPopupContent.extend({
   defaults: {
      'bRefreshOnClose': false,
      'bIsCombinedOp': false,
      'bUseMaxHeight': false
   }
});

var VAlertPopup = AVPopupContent.extend({
   
   render: function() {

      this.drawHeader("STR_ALERT");

      // Body
      var oMAlert = new MAlert({
         'sMessage': this.model.get('sMessage')
      });
      this.oVPopupBody = new VAlert({ 
         'className':'popupBody', 
         'model': oMAlert
      });
      this.$el.append(this.oVPopupBody.$el);
      
      // Footer
      this.drawFooter();
      
      return this;
   },

   drawFooter: function() {
      var oFooter = $('<div/>', { 'class': 'popupFooter' });
      var oInfoDiv = $('<div/>', { 'class':"info" });
      var oCloseButton = $('<a/>', { 
         'href': '',
         'class': 'button pianoButton op default Dismiss popupTabbable' 
      });
      oCloseButton.append(TranslateString("STR_OK"));
      var oButtons = $('<div/>', { 'class': 'buttons' });
      oButtons.append(oCloseButton);
      oFooter.append(oInfoDiv);
      oFooter.append(oButtons);
      this.$el.append(oFooter);
   },

   onDismiss: function() {
      if (this.model.get('bRefreshOnClose'))
      {
         this.$el.trigger("refreshPage");
      }
      // Close popup.
      this.$el.trigger("closePopup");
      return false;
   }
   
});

var MHelpPopup = AMPopupContent.extend({
   defaults: {
      'bUseMaxHeight': true
   }
});

var VHelpPopup = AVPopupContent.extend({
   
   render: function() {
      var sSelectedTitle = this.model.get('sSelectedTitle');

      var oHeader = this.drawHeader("STR_HELP");

      // Add search box for help
      var oSearchBoxWrapper = $('<div/>', {
         'class':'helpSearchBoxWrapper'
      });
      var oSearchIcon = $('<span/>', {
         'class':'searchIcon HPSSA_SPRITE',
         'id':'searchIcon'
      });
      var oSearchBoxSpan = $('<span/>', {
         'class':'helpSearchBoxSpan'
      });
      var oSearchBox = $('<input/>', {
         'class':'helpSearchBox popupTabbable empty',
         'type':'text',
         'value':TranslateString("HELP_SEARCH")
      });
      var oClearSearchBox = $('<span/>', {
         'class':'clearSearchBoxButton HPSSA_SPRITE',
         'id':'clearButton'
      });
      oSearchBoxSpan.append(oSearchIcon);
      oSearchBoxSpan.append(oSearchBox);
      oSearchBoxSpan.append(oClearSearchBox);
      oSearchBoxWrapper.append(oSearchBoxSpan);

      oHeader.append(this.getDividerDiv());
      oHeader.append(oSearchBoxWrapper);

      // Draw navigation buttons
      var oNavDiv = this.drawHeaderNav(oHeader);

      // Body
      var oMHelpBuilder = new MHelpBuilder({
         'sSelectedTitle':sSelectedTitle,
         'oNavForwardButton':this.$('.navForward'),
         'oNavBackwardButton':this.$('.navBackward'),
         'oSearchBoxWrapper':oSearchBoxWrapper,
         'oSearchBox':oSearchBox,
         'oClearSearchBox':oClearSearchBox
      });
      this.oVPopupBody = new VHelpBuilder({
         'className':'popupBody',
         'model':oMHelpBuilder
      });
      this.$el.append(this.oVPopupBody.$el);
      
      // Footer
      this.drawFooter();
      var oHPSSAVersion = g_oGlobals.Version;
      var oSOULAPIVersion = g_oGlobals.SOULAPIVersion;
      var oINFOMGRVersion = g_oGlobals.INFOMGRVersion;
      this.drawFooterInfoSection("STR_HPSSA",oHPSSAVersion);
      //this.drawFooterInfoSection("STR_SOULAPI",oSOULAPIVersion);
      //this.drawFooterInfoSection("STR_INFOMGR",oINFOMGRVersion);
      this.drawVersionInfoSection();
      
      return this;
   },

   drawVersionInfoSection: function() {
      var oHPSSAVersion = g_oGlobals.Version;
      var oSOULAPIVersion = g_oGlobals.SOULAPIVersion;
      var oINFOMGRVersion = g_oGlobals.INFOMGRVersion;
      var oInfoDiv = this.$('.popupFooter .info');
      var oSectionDiv = $('<div/>', {
         'class':'section'
      });

      /*var oLabelSpan = $('<span/>', {
         'class':'label'
      });
      var oValueSpan = $('<span/>', {
         'class':'value'
      });
      oLabelSpan.html(TranslateString("STR_HPSSA") + ": ");
      oValueSpan.html(TranslateString(oHPSSAVersion) + " ");
      oSectionDiv.append(oLabelSpan);
      oSectionDiv.append(oValueSpan);*/

      var oLabelSpan2 = $('<span/>', {
         'class':'label'
      });
      var oValueSpan2 = $('<span/>', {
         'class':'value'
      });
      oLabelSpan2.html(TranslateString("STR_SOULAPI") + ": ");
      oValueSpan2.html(TranslateString(oSOULAPIVersion) + " ");
      oSectionDiv.append(oLabelSpan2);
      oSectionDiv.append(oValueSpan2);

      if ( typeof(oINFOMGRVersion) != "undefined" )
      {
         var oLabelSpan3 = $('<span/>', {
            'class':'label'
         });
         var oValueSpan3 = $('<span/>', {
            'class':'value'
         });
         oLabelSpan3.html(TranslateString("STR_INFOMGR") + ": ");
         oValueSpan3.html(TranslateString(oINFOMGRVersion));
         oSectionDiv.append(oLabelSpan3);
         oSectionDiv.append(oValueSpan3);
      }
      oInfoDiv.append(oSectionDiv);
   },

   resizeWidth: function() {
      var nWidth = this.model.get('nWidth');
      var nTitleWidth = this.$('.deviceLabel').width() + 31; //30 is for margins
      var nNavWidth = this.$('.headerNavDiv').width();
      this.$('.helpSearchBoxWrapper').width(nWidth - nTitleWidth - nNavWidth);
   }

});

var MDiagsPopup = AMPopupContent.extend({
   defaults: {
      'bUseMaxHeight': true
   }
});

var VDiagsPopup = AVPopupContent.extend({

   events: {
      'click .button.pianoButton.Save.op': 'onSave',
      'click .button.pianoButton.Dismiss.op': 'onDismiss',
      'click .button.pianoButton.BackToDisplay.op': 'backToDisplay',
      'click .button.pianoButton.RefreshDevices.op': 'refreshDevices',
      'click .button.pianoButton.OfflineSave.op': 'saveToRemovableDevice',
      'keydown input#textBox' : 'checkValidInput'
   },

   initialize: function() {
      AVPopupContent.prototype.initialize.call(this);
      this.oVPopupBody.model.on("change:bReportIsFinished", this.reportIsFinished, this);
   },
   
   render: function() {
      var sDeviceID = this.model.get('sDeviceID');
      var sReportType = this.model.get('sReportType');
      var bDisplay = this.model.get('bDisplay');
      var sLabel = '';
      var sIconClass = '';

      if (sReportType == "ADU") {
         sLabel = "STR_DIAG_GENERATING_REPORT";
         sIconClass = 'HPSSA_SPRITE icon ICON_DIAGNOSTICS';
      }
      else if (sReportType == "SSD") {
         sLabel = "STR_SSD_GENERATING_REPORT";
         sIconClass = 'HPSSA_SPRITE icon ICON_SSD_WEAR_GAUGE';
      }

      this.drawHeader(sLabel, sIconClass);

      // Body
      var oMArrayDiagnosticsReport = new MArrayDiagnosticsReport({
         'sDeviceID': sDeviceID,
         'sOpName': "GeneratingReport",
         'sReportType': sReportType,
         'bDisplay': bDisplay
      });
      this.oVPopupBody = new VArrayDiagnosticsReport({ 
         'className':'popupBody', 
         'model': oMArrayDiagnosticsReport
      });
      this.$el.append(this.oVPopupBody.$el);

      // Footer
      this.drawFooter();
      this.drawButtons();
      
      return this;
   },

   onDismiss: function() {
      if (!this.oVPopupBody.model.get('bReportIsFinished')) {
         CancelReport();
      }
      AVPopupContent.prototype.onDismiss.call(this);
      return false;
   },

   getDefaultButtons: function() {
      // TEMP FIX for Enter not clicking the correct button in IE.
      // This "disables" the Enter button while bReportIsFinished is false
      var bReportIsFinished = this.model.get('bReportIsFinished');
      var oBtn;
      if (!bReportIsFinished) {
         oBtn = [];
      }
      else {
         oBtn = this.$('.default');
      }
      return oBtn;
      //return this.$('.default');
   },

   drawFooter: function() {
      var oFooter = $('<div/>', { 'class': 'popupFooter' });
      var oInfoDiv = $('<div/>', { 'class':"info" });
      var oLoadingDiv = $('<div/>', { 'class':"loading" });
      var oButtons = $('<div/>', { 'class': 'buttons' });
      oFooter.append(oInfoDiv);
      oFooter.append(oLoadingDiv);
      oFooter.append(oButtons);
      this.$el.append(oFooter);
   },

   drawButtons: function() {
      this.hideLoadingIcon();
      var oVPopupBody = this.oVPopupBody;
      var oButtons = this.$('.popupFooter .buttons');
      oButtons.empty();
      if (this.model.get('bReportIsFinished'))
      {
         var sLabel = 'STR_SAVE';
         var sClass = "button pianoButton default op Save popupTabbable";
         var oButton = $('<a/>', {
            'href': '',
            'id': sLabel,
            'class': sClass
         });
         oButton.html(TranslateString(sLabel));
         oButtons.append(oButton);
         var sLabel2 = 'STR_CLOSE';
         var sClass2 = "button pianoButton op Dismiss popupTabbable";
         var oButton2 = $('<a/>', {
            'href': '',
            'id': sLabel2,
            'class': sClass2
         });
         oButton2.html(TranslateString(sLabel2));
         oButtons.append(oButton2);
      }
      else
      {
         var sLabel = 'STR_CANCEL';
         var sClass = "button pianoButton op default Dismiss CancelReport popupTabbable";
         var oButton = $('<a/>', {
            'href': '',
            'id':sLabel,
            'class':sClass
         });
         oButton.html(TranslateString(sLabel));
         oButtons.append(oButton);
      }
   },

   onSave: function() {
      var bIsOfflineEnvironment = g_oGlobals.OperationMode == "Offline";
      if (bIsOfflineEnvironment) {
         this.enumerateRemovableDevices();
         //this.drawOfflineSaveDialog();
      }
      else {
         var oReportDL = GetReportContentForDownload();
         document.location = "ADUReport.zip";   
      }
      return false;
   },

   reportIsFinished: function() {
      var oVPopupBody = this.oVPopupBody;
      if (oVPopupBody.model.get('bReportIsFinished')) {
         // Update progress bars
         this.model.set({'bReportIsFinished':true})
         this.updateHeader();
         this.drawButtons();
         var o = oVPopupBody.model.get('oProgress');
         oVPopupBody.UpdateOverallProgress(4,o);
         sReportType = oVPopupBody.model.get('sReportType');
         var sLabel;
         var sIconClass;
         if (this.model.get('bDisplay')) {
            if (sReportType == "SSD") {
               sIconClass = 'HPSSA_SPRITE icon ICON_SSD_WEAR_GAUGE';
               sLabel = 'STR_SSD_WEAR_GAUGE_REPORT';
               this.drawSSDReport();
            }
            else if (sReportType == "ADU") {
               sIconClass = 'HPSSA_SPRITE icon ICON_DIAGNOSTICS';
               sLabel = 'STR_DIAGNOSTIC_REPORT';
               var oReportDiv = $('<div/>', {'class':'reportDiv'});
               var sReport = GetReportContentForDisplay("HTML");
               oReportDiv.html(sReport);
               this.oVPopupBody.$el.empty();
               this.oVPopupBody.$el.append(oReportDiv);
            }
         }
         else {
            this.onSave();
            var bIsOfflineEnvironment = g_oGlobals.OperationMode == "Offline";
            if (!bIsOfflineEnvironment) {
               this.onDismiss();
            }
         }
      }
   },

   drawSSDReport: function() {
      var oOldDiv = this.$('.popupBody');
      oOldDiv.remove();
      var oWearGaugeWrapperDiv = $('<div/>', {
         'class': 'wearGaugePopupWrapper'
      });
      var oReport = GetReportContentForDisplay("SSD");

      var oModel = new MWearGaugeTreeActions({
         'oReport': oReport.SmartSSDWearGaugeReport
      });
      this.oVPopupBody = new VWearGaugeTreeActions({
         'className': 'popupBody wearGaugePopupWrapper',
         'model': oModel
      });
      this.$('.popupHeader').after(this.oVPopupBody.$el);
      this.resize();
   },

   enumerateRemovableDevices: function() {
      this.showLoadingIcon();
      this.$('.popupFooter .buttons').hide();
      var oThis = this;
      EnumerateRemovableDevices(function(o) {
         oThis.drawOfflineSaveDialog(o);
      });
   },

   drawOfflineSaveDialog: function(o) {
      // Enumerate all removable devices
      var oRemovableDevices = o.RequestResult;
      //var oRemovableDevices = [];
      // Hide all children of .popupBody
      var oPopupBody = this.$('.popupBody');
      oPopupBody.children().hide();
      // Draw wrapper
      var oOfflineSaveWrapper = $('<div/>', {'class': "offlineSaveWrapper"});
      oPopupBody.append(oOfflineSaveWrapper);
      var oBodyPadding = $('<div/>', {'class': "bodyPadding"});
      oOfflineSaveWrapper.append(oBodyPadding);
      var oNotesDiv = $('<div/>', {'class': "notes"});
      oOfflineSaveWrapper.append(oNotesDiv);
      var oNotes = [];
      // If there are no removable devices, then display a note
      if (oRemovableDevices.length == 0) {
         oNotes.push({
            'NoteType':'CONFIRM',
            'NoteContent':['STR_NO_REMOVABLE_MEDIA']
         });
         var oMCollapsedNote = new MCollapsedNote({
            'nDisplay':'0',
            'oNotes':oNotes
         });
         var oVCollapsedNote = new VCollapsedNote({
            'model': oMCollapsedNote
         });
         oNotesDiv.append(oVCollapsedNote.$el);
         this.drawOfflineSaveButtons(false, false)
      }
      else {
         // Save to root/filename note
         oNotes.push({
            'NoteType':'NOTE',
            'NoteContent':['STR_DIAG_NOTE_SAVE_TO_ROOT']
         });
         oNotes.push({
            'NoteType':'NOTE',
            'NoteContent':['STR_DIAG_NOTE_FILENAME']
         });
         var oMCollapsedNote = new MCollapsedNote({
            'nDisplay':'0',
            'oNotes':oNotes
         });
         var oVCollapsedNote = new VCollapsedNote({
            'model': oMCollapsedNote
         });
         oNotesDiv.append(oVCollapsedNote.$el);
         // Radio control for selecting a removable device
         var oSelectWrapper = $('<div/>', {'class': "control"});
         var oSelectTitleDiv = $('<div/>', {'class': "title"});
         oSelectTitleDiv.html(TranslateString("STR_SELECT_REMOVABLE_DEVICE"));
         var oRadioDiv = $('<div/>', {'class': "controlWrapper"});
         for (var i=0; i < oRemovableDevices.length; ++i) {
            var oRadioInputDiv = $('<div/>', {'class': "radioInputWrapper"});
            var sID = "device" + i;
            var oRadioInput = $('<input/>', {
               'id': sID,
               'class': "radioInput",
               'type': "radio",
               'name': "device",
               'value': oRemovableDevices[i].RemovableDeviceID
            });
            var oRadioInputLabel = $('<label/>', {
               'for': sID,
               'class': "radioInputLabel"
            });
            oRadioInputLabel.html(TranslateString(oRemovableDevices[i].RemovableDeviceDescription));
            oRadioInputDiv.append(oRadioInput);
            oRadioInputDiv.append(oRadioInputLabel);
            oRadioDiv.append(oRadioInputDiv);
            if (i == 0) {
               oRadioInput.prop("checked", true);
            }
         }
         oSelectWrapper.append(oSelectTitleDiv);
         oSelectWrapper.append(oRadioDiv);
         oOfflineSaveWrapper.append(oSelectWrapper);
         // Text control for entering a filename
         var oNameWrapper = $('<div/>', {'class': "control"});
         var oNameTitleDiv = $('<div/>', {'class': "title"});
         oNameTitleDiv.html(TranslateString("STR_OFFLINE_SAVE_NAME"));
         var oNameDiv = $('<div/>', {'class': "controlWrapper"});
         var oTextBox = $('<input/>',
         {
            'id': "textBox",
            'class': "textInput popupTabbable",
            'type': "text",
            'name': "filename",
            'value': "",
            'width': "200px",
         });
         var oTextBoxLabel = $('<label/>', {
            'for': "textBox",
            'class': "zipLabel"
         });
         oTextBoxLabel.html(".zip");
         oNameDiv.append(oTextBox);
         oNameDiv.append(oTextBoxLabel);
         oNameWrapper.append(oNameTitleDiv);
         oNameWrapper.append(oNameDiv);
         oOfflineSaveWrapper.append(oNameWrapper);
         this.drawOfflineSaveButtons(true);
      }
   },

   drawOfflineSaveButtons: function(bDevicesFound, bIsFinishedSaving) {
      this.hideLoadingIcon();
      var bDisplay = this.model.get('bDisplay');
      var oVPopupBody = this.oVPopupBody;
      var oButtons = this.$('.popupFooter .buttons');
      oButtons.empty();

      // Show: Refresh Devices if there are no devices
      //       Save to Removable Media if there are
      // Show: Back if bDisplay is true
      //       Close if bDisplay is false

      var showSaveButton = false;
      var showRefreshButton = false;
      var showBackButton = false;
      var showCloseButton = false;

      if (bDevicesFound) {

      }
      if (bIsFinishedSaving) {
         if (bDisplay) {
            this.drawButton(oButtons, "STR_BACK", "default BackToDisplay");
         }
         else {
            this.drawButton(oButtons, "STR_BACK", "default RefreshDevices");
         }
         this.drawButton(oButtons, "STR_CLOSE", "Dismiss");
      }
      else if (bDevicesFound) {
         this.drawButton(oButtons, "STR_SAVE_TO_REMOVABLE_DEVICE", "default OfflineSave");
         this.drawButton(oButtons, "GUI_OP_REFRESH_REMOVABLE_DEVICES", "RefreshDevices");
         if (bDisplay) {
            this.drawButton(oButtons, "STR_BACK", "BackToDisplay");
         }
         else {
            this.drawButton(oButtons, "STR_CLOSE", "Dismiss");
         }
      }
      else {
         this.drawButton(oButtons, "GUI_OP_REFRESH_REMOVABLE_DEVICES", "default RefreshDevices");
         if (bDisplay) {
            this.drawButton(oButtons, "STR_BACK", "BackToDisplay");
         }
         else {
            this.drawButton(oButtons, "STR_CLOSE", "Dismiss");
         }
      }
   },

   drawButton: function(oButtons, sLabel, sClass) {
      var sLabel = sLabel;
      var sClass = "button popupTabbable pianoButton op " + sClass;
      var oButton = $('<a/>', {
         'href': '',
         'id':sLabel,
         'class':sClass
      });
      oButton.html(TranslateString(sLabel));
      oButtons.append(oButton);
   },

   refreshDevices: function() {
      this.enumerateRemovableDevices();
      return false;
   },

   saveToRemovableDevice: function() {
      var sSelectedDrive = this.$('input[name=device]:checked').val();
      var sFilename = this.$('input[name=filename]').val() + ".zip";
      this.showLoadingIcon();
      this.$('.popupFooter .buttons').hide();
      var oThis = this;
      WriteReportToRemovableDevice(sSelectedDrive, sFilename, function(o) {
         oThis.saveCallback(o);
      });
      return false;
   },

   saveCallback: function(o) {
      this.hideLoadingIcon();
      // Get save information
      var oSaveResult = o.RequestResult;
      var bSaveSucceeded = oSaveResult.SaveSucceeded;
      var sSaveStatusMessage = oSaveResult.SaveStatusMessage;
      var nSavedReportSize = oSaveResult.SavedReportSize;
      // Empty display and rebuild
      var oOfflineSaveWrapper = this.$('.offlineSaveWrapper');
      oOfflineSaveWrapper.empty();
      var oBodyPadding = $('<div/>', {'class': "bodyPadding"});
      oOfflineSaveWrapper.append(oBodyPadding);
      var oNotesDiv = $('<div/>', {'class': "notes"});
      oOfflineSaveWrapper.append(oNotesDiv);
      // Create the note
      var sNoteType;
      if (bSaveSucceeded) { sNoteType = "NOTE"; }
      else { sNoteType = "CONFIRM"; }
      var oNotes = [];
      oNotes.push({
         'NoteType':sNoteType,
         'NoteContent':[sSaveStatusMessage]
      });
      var oMCollapsedNote = new MCollapsedNote({
         'nDisplay':'0',
         'oNotes':oNotes
      });
      var oVCollapsedNote = new VCollapsedNote({
         'model': oMCollapsedNote
      });
      oNotesDiv.append(oVCollapsedNote.$el);
      this.drawOfflineSaveButtons(true, true)
   },

   backToDisplay: function() {
      this.$('.offlineSaveWrapper').remove();
      var oPopupBody = this.$('.popupBody');
      oPopupBody.children().show();
      this.drawButtons();
      return false;
   },

   checkValidInput: function(e) {
      var key = e.which || e.keyCode;
      var isValidChar = false;
      if ( !e.shiftKey && !e.altKey && !e.ctrlKey &&
         // numbers   
         key >= 48 && key <= 57 ||
         // Numeric keypad
         key >= 96 && key <= 105 ||
         // Ctrl-a
         e.ctrlKey && key == 65 ||
         // comma, period and minus, . on keypad
         key == 190 || key == 188 || key == 109 || key == 110 ||
         // Backspace and Tab and Enter
         key == 8 || key == 9 || key == 13 ||
         // Home and End
         key == 35 || key == 36 ||
         // left and right arrows
         key == 37 || key == 39 ||
         // Del and Ins
         key == 46 || key == 45 )
      {
         isValidChar = true;
      }
      // allow letters
      else if ( key >= 65 && key <= 90 )
      {
         isValidChar = true;
      }
      return isValidChar;
   },

   updateHeader: function() {
      var sDeviceID = this.model.get('sDeviceID');
      var oPopupHeader = this.$('.popupHeader');
      oPopupHeader.empty();
      var oDeviceLabelDiv = $('<div/>', {'class': 'deviceLabel'});
      var oMDeviceLabelOptions;
      if (sDeviceID == "proliant") {
         oMDeviceLabelOptions = {
            'sDeviceID': sDeviceID,
            'sLabel': TranslateString("STR_HP_PROLIANT"),
            'sIconClass': "HPSSA_SPRITE ICON_SERVER"
         };
      }
      else {
         oMDeviceLabelOptions = {
            'sDeviceID': sDeviceID
         };
      }
      var oMDeviceLabel = new MDeviceLabel(oMDeviceLabelOptions);
      var oVDeviceLabel = new VDeviceLabel({
         'model': oMDeviceLabel,
         'el': oDeviceLabelDiv
      });
      oPopupHeader.append(oDeviceLabelDiv);
      return oPopupHeader;
   },

   resizeWidth: function() {
      AVPopupContent.prototype.resizeWidth.call(this);
      var nWidth = this.model.get('nWidth');
      this.oVPopupBody.model.set({
         'nWidth':nWidth
      });
   },

   resizeHeight: function() {
      AVPopupContent.prototype.resizeHeight.call(this);
   }

});