/* 
Smart Storage Administrator View
- vhome.js
- vconfiguration.js
- varraydiagnostics.js

Model:
- MSmartStorageAdministrator
*/
var VSmartStorageAdministrator = AVResize.extend({

   // DOM Events
   events: {
      "click #AppRefreshLink": "clickRescan",
      "click #AppHelpLink": "clickHelp",
      "click #hp-main-menu-control": "clickMainMenu",
      //"click a.link": "clickLink",
      "click #AppExitLink": "clickExit",
      "refreshPage": "refresh",
      "popup": "onPopup",
      "closePopup": "onClosePopup",
      "focus a": "onFocus",
      "focus input": "onFocus"
   },
   
   // 1. On page refresh, this is the first method that gets called.
   // 2. AVBody
   refresh: function(e) {
      if (IsDefined(this.oBV))
      {
         this.oBV.refresh();
      }
   },

   // Returns true or false if there is an open popup.
   isPopupOpen: function(e) {
      return IsDefined(this.oVPopup);
   },

   onFocus: function(e) {
      var o = $(e.currentTarget);
      // Check if there's a popup.
      if (this.isPopupOpen())
      {
         // Only allow tabbing within the popup.
         if (!o.hasClass("popupTabbable"))
         {
            var oVisiblePopupTabbables = this.$('.popupTabbable:visible');
            if (oVisiblePopupTabbables.length > 0)
            {
               var oNextInput = oVisiblePopupTabbables.get(0);
               if (IsDefined(oNextInput))
               {
                  oNextInput.focus();
               }
            }
            // If no tabbable items are found in the popup, then don't do anything.
            else
            {
               return false;
            }
         }
      }
   },

   // Initialize SSA View
   initialize: function() {
      AVResize.prototype.initialize.call(this);
      this.oBV = null;
      this.oVPopup = null;
      g_oRSSA.on("route:home", this.clickHome, this);
      g_oRSSA.on("route:diagnostics", this.clickDiagnostics, this);
      g_oRSSA.on("route:configuration", this.clickConfiguration, this);
      g_oRSSA.on("route:exit", this.clickExit, this);
      this.render();
      // If the current state is not ready, then add event binding to be
      // notified when it is ready.
      if (this.model.get("nState") == 0)
      {
         this.model.bind("change:nState", this.checkState, this);
      }
      else
      {
         this.checkState();
      }
   },

   dispose: function() {
      this.disposeBV();
      this.disposePopup();
   },

   disposeBV: function() {
      if (IsDefined(this.oBV))
      {
         this.oBV.dispose();
         this.oBV = null;
      }
   },

   disposePopup: function() {
      if (this.isPopupOpen())
      {
         this.oVPopup.dispose();
         this.oVPopup = null;
      }
   },

   render: function() {
      var sAppName = TranslateString("STR_SMART_STORAGE_ADMINISTRATOR_TOP");
      var sAppNameBottom = TranslateString("STR_SMART_STORAGE_ADMINISTRATOR_BOTTOM");
      var sLabelClass = "label";
      if (IsDefined(sAppNameBottom) && sAppNameBottom != '')
      {
         sAppName += "<br/>" + sAppNameBottom;
         //sLabelClass = "labelTwoLines";
      }
      // Only show exit link in offline mode, since online will provide own window exit button.
      //var sExitLink = g_oGlobals.OperationMode == "Online" ? '' : '<a href="#exit" id="AppExitLink" class="headerLink HPSSA_SPRITE" title="' + TranslateString("STR_EXIT") + '"></a>';
      // show exit link in all modes for now
      var sExitLink = '<a href="" id="AppExitLink" class="headerLink enabled" title="' + TranslateString("STR_EXIT") + '"><div class="HPSSA_SPRITE"></div></a>';
      
      var sHTML = 
         '<header id="hp-banner-container">' +
            '<div class="hp-header-primary">' +
               '<a href="" id="hp-main-menu-control" class="headerLink popupable enabled">' +
                  '<div class="logo" id="hp-logo" title="HP Smart Storage Administrator"><div class="HPSSA_SPRITE"></div></div>' +
                  '<div class="' + sLabelClass + '">' + sAppName + '</div>' +
                  '<div class="arrow HPSSA_SPRITE"></div>' +
               '</a>' +
            '</div>' +
            '<div class="hp-header-secondary">' +
               //'<a href="#home" id="AppHomeLink" class="appBodyLink headerLink HPSSA_SPRITE" title="' + TranslateString("STR_HOME") + '"></a>' +
               //'<a href="#refresh" id="AppRefreshLink" class="appBodyLink headerLink HPSSA_SPRITE" title="' + TranslateString("STR_REFRESH") + '"></a>' +
               //'<a href="#settings" id="AppSettingsLink" class="appBodyLink headerLink HPSSA_SPRITE" title="' + TranslateString("STR_SETTINGS") + '"></a>' +
               //'<a href="#help" id="AppHelpLink" class="appBodyLink headerLink HPSSA_SPRITE modal" title="' + TranslateString("STR_HELP") + '"></a>' +
               '<a href="" id="AppRefreshLink" class="headerLink popupable enabled modal" title="' + TranslateString("STR_RESCAN") + '"><div class="HPSSA_SPRITE"></div></a>' +
               '<a href="" id="AppHelpLink" class="headerLink popupable enabled modal" title="' + TranslateString("STR_HELP") + '"><div class="HPSSA_SPRITE"></div></a>' +
               sExitLink +
            '</div>' +
            '<div class="clearFloats"></div>' + 
         '</header>' +
         '<div id="appBody"></div>';
      $(this.el).html(sHTML);
      this.drawLoading();
      $(this.el).disableSelection();
      return this;
   },/*
   
   clickLink: function(e) {
      NavigateTo(($(e.currentTarget).attr("href")), true, false, true);
      TriggerResize();
      return false;
   },*/

   drawLoading: function() {
      this.disableHeader();
      var oBody = this.$('#appBody');
      var oLoadingDiv = $('<div/>', {
         'class': 'loading'
      });
      var oIconDiv = $('<div/>', {
         'class': 'HPSSA_ICON_LOADING HPSSA_SPRITE'
      });
      var oLabelDiv = $('<div/>', {
         'class': 'label'
      });
      oLabelDiv.html(TranslateString("STR_LOADING"));
      oLoadingDiv.append(oIconDiv);
      oLoadingDiv.append(oLabelDiv);
      oBody.html(oLoadingDiv);
      TriggerResize();
   },

   disableHeader: function() {
      this.model.set({
         'bHeaderEnabled':false
      });
      var oHeaderLink = this.$('.headerLink');
      oHeaderLink.addClass('disabled');
      oHeaderLink.removeClass('enabled');
   },
   
   enableHeader: function() {
      this.model.set({
         'bHeaderEnabled':true
      });
      var oHeaderLink = this.$('.headerLink');
      oHeaderLink.addClass('enabled');
      oHeaderLink.removeClass('disabled');
   },
   
   checkState: function(e) {
      var nState = this.model.get('nState');
      if (nState == 1)
      {
         this.enableHeader();
         Backbone.history.start();
      }
   },
   
   loadBV: function(sID) {
      var sClass = "main";
      // Remove existing Body View.
      if (this.oBV)
      {
         // Dispose of the AVBody view.
         this.oBV.dispose();
      }
      var oDiv = $('<div id="' + sID + '" class="' + sClass + '"></div>');
      this.$('#appBody').html(oDiv);
      return oDiv;
   },
   
   // Refresh the application (Rescan).
   // This is called when either:
   // 1. The refresh button in the main menu navigation is clicked, or
   // 2. The browser refresh button/F5 shortcut key is pressed.
   clickRescan: function() {
      //NavigateTo(Backbone.history.fragment);
      if ( confirm(TranslateString("STR_MSG_RESCAN_WARNING")) )
      {
         // Let the server know we're abandoning the application
         window.onunload = null;
         ExitApplication();	  
         location.reload();
      }
      return false;
   },

   // Called when "r" keyboard control is pressed.
   clickRefresh: function() {
      // Do not allow refresh to be triggered if there's an open popup.
      if (!this.isPopupOpen())
      {
         var oRefreshBtn = this.$('#refreshButtonID');
         if (oRefreshBtn.length == 1)
         {
            oRefreshBtn.click();
         }
      }
      return false;
   },
   
   clickHome: function(sTab1, sID, sTab2) {
      // Add "Home" to URL.
      //this.drawLoading();
      
      // Navigate Home after updating oControllerList.
      //this.model.initializeControllerList();
      //g_oRSSA.navigate("home", {trigger: false, replace: true});
      var oModel = new MHome({
         'oControllerList': this.model.get('oControllerList'),
         'sDeviceID':sID,
         'sTab':sTab1,
         'sTab1':sTab2
      });
      var sLabel = "STR_HOME";
      this.oBV = new VHome({
         'el': this.loadBV(sLabel), 
         'model': oModel
      });
      var oMenuLinks = this.$('.main-menu-a');
      oMenuLinks.removeClass('hp-selected');
      this.model.set('sSelected', sID);
      this.postClickBV();
   },
   
   clickDiagnostics: function(sID, sTab, sTab1, sTab2) {
      var oModel = new MDiagnostics({
         "sDeviceID": sID,
         "sTab": sTab, 
         "sTab1": sTab1,
         "sTab2": sTab2
      });
      this.oBV = new VDiagnostics({ 
         el: this.loadBV("STR_DIAGNOSTICS"),
         model: oModel
      });
      this.model.set('sSelected', sID);
      this.postClickBV();
   },/*
   
   clickSettings: function() {
      if (this.model.get('bHeaderEnabled'))
      {
         var oModel = new MSettings();
         this.oBV = new VSettings({
            'el': this.loadBV("VSettingsDiv"), 
            'model': oModel
         });
         this.postClickBV();
      }
   },*/
   
   clickConfiguration: function(sID, sTab, sTab1, sTab2) {
      var oDeviceID = GetDeviceIDSection(sID);
      var sLabel = TranslateString(oDeviceID.LABEL);   
      var oModel = new MConfiguration({
         "sDeviceID": sID, 
         "sTab": sTab, 
         "sTab1": sTab1,
         "sTab2": sTab2
      });
      this.oBV = new VConfiguration({
         el: this.loadBV("STR_CONFIGURATION", sLabel),
         model: oModel
      });
      
      // Update selection in main menu.
      // First remove selected from any previously selected.
      this.model.set('sSelected', sID);
      this.postClickBV();
   },
   
   drawVersion: function(sLabel, sValue, oParentDiv) {
      var oDiv = $('<div/>', { 'class':'version' });
      var oLabelSpan = $('<span/>', { 'class':'label' });
      var oValueSpan = $('<span/>', { 'class':'value' });
      oLabelSpan.append(TranslateString(sLabel) + ": ");
      oValueSpan.append(sValue);
      oDiv.append(oLabelSpan);
      oDiv.append(oValueSpan);
      oParentDiv.append(oDiv);
   },

   drawHelpHeader: function(sTitle, oParentDiv) {
      var oHeader = $('<div/>', { 'class': 'helpHeader' });
      oHeader.append(TranslateString(sTitle));
      oParentDiv.append(oHeader);
   },

   clickMainMenu: function() {
      if (this.model.get('bHeaderEnabled'))
      {
         this.$('#hp-main-menu-control').addClass('selected');
         var oMMainMenuPopup = new MMainMenuPopup({
            'sSelected': this.model.get('sSelected'),
            'oControllerList': this.model.get('oControllerList')
         });
         var oVMainMenuPopup = new VMainMenuPopup({
            'model': oMMainMenuPopup
         });
         this.$el.trigger("popup", {
            'sType': 'mainMenuPopup',
            'oVContent': oVMainMenuPopup,
            'bClickOutsideToClose':true
         });
      }
      return false;
   },

   clickHelp: function() {
      if (this.model.get('bHeaderEnabled'))
      {
         var sSelectedTitle = "STR_GETTING_STARTED_WITH_SSA_OVERVIEW";
         var oMHelpPopup = new MHelpPopup({
            'sSelectedTitle':sSelectedTitle
         });
         var oVHelpPopup = new VHelpPopup({
            'model':oMHelpPopup
         });
         this.$el.trigger("popup", {
            'sType': 'helpPopup',
            'oVContent': oVHelpPopup,
            'bClickOutsideToClose':true
         });
      }
      return false;
   },

   clickExit: function() {
      var bIsOfflineEnvironment = g_oGlobals.OperationMode == "Offline";
      var bAltExitPrompt = g_oGlobals.AltExitPrompt;
      var sWarningString = bAltExitPrompt ? "STR_WARNING_EXIT_REBOOT" : "STR_WARNING_EXIT";
      var exit = confirm(TranslateString(sWarningString));
      
      // Display exit confirmation.
      // TODO: Implement confirm popup.
      /*var oMConfirmPopup = new MConfirmPopup({
         'sMessage': sWarningString
      });
      var oVConfirmPopup = new VConfirmPopup({
         'model': oMConfirmPopup
      });
      this.$el.trigger("popup", {
         'sType': 'confirmPopup',
         'oVContent': oVConfirmPopup
      });*/

      if (exit)
      {
         // Remove the onunload handlers
         window.onunload = null;
         ExitApplication();
         if (bIsOfflineEnvironment)
         {      
            // TODO: Turn off all the LED's
            //g_blinkManager.BlinkAsync(g_dropDown.getCurrentSelectedValue(), true);
         }
         if(g_isMoz)
         {
            // This next line is a Firefox hack. Without it, Firefox will not 
            // close the window because it does not allow windows that were
            // not created with script to be closed by script.

            // For Firefox (not verified)
            window.open('','_parent','');
            window.close();

            // In case window is still opened:
            // Verified to work in:
            // Chrome
            window.open('', '_self');
         }
         else if (msieversion() >= 6)
         {
            // Verified to work in:
            // IE 10
            window.opener = 'Self';
            window.open('','_parent');
         }
         else
         {
            window.open('', '_self');
         }
         window.close();
      }
      return false;
   },

   postClickBV: function() {
      TriggerResize();
      this.onClosePopup();
   },

   onPopup: function(e, options) {
      //this.disposePopup();
      // Make sure no popups are open.
      if (!this.isPopupOpen())
      {
         var sType = options.sType;
         var oModelOptions = {};
         if (sType == 'actionConfigPopup' || 
             sType == 'detailsPopup' ||
             sType == 'statusPopup' ||
             sType == 'helpPopup' ||
             sType == 'diagsPopup')
         {
            this.disableHeader();
            oModelOptions = {
               'oVContent': options.oVContent,
               'nPopupWidth': 900,
               'sCustomInnerID': "actionPopupID",
               'sMinHeight': '578px',
               'bClickOutsideToClose': options.bClickOutsideToClose,
               'sType': sType
            };
         }
         else if (sType == 'mainMenuPopup')
         {
            oModelOptions = {
               'oVContent': options.oVContent,
               'bUseWindowWidth': true,
               'sAlign': 'left',
               'nPopupWidth': this.model.get('nWidth'),
               'nTop': 60,
               //'sPadding': '45px',
               'sBorderWeight': '0 1px', // the border around the popup.
               'nBorderRadius': 0,
               'nDisableOpacity': 0,
               'sCustomInnerID': "menuPopup",
               'sMinHeight': '100px',
               'bClickOutsideToClose': options.bClickOutsideToClose,
               'sType': sType
            };
         }
         else if (sType == "alertPopup")
         {
            oModelOptions = {
               'oVContent': options.oVContent,
               'nPopupWidth': 300,
               'sCustomInnerID': "alertPopupID",
               'sMinHeight':'30px',
               'bClickOutsideToClose': options.bClickOutsideToClose,
               'sType': sType
            };
         }
         var oMPopup = new MPopup(oModelOptions);
         this.oVPopup = new VPopup({
            'model': oMPopup
         });
         this.$el.append(this.oVPopup.$el);
         if (sType == 'actionConfigPopup')
         {
            // Set focus to the first action control.
            this.oVPopup.setFocus();
         }
         this.resize();
      }
   },

   onClosePopup: function(e) {
      this.disposePopup();

      // If main menu was closed, remove selected from its class.
      this.$('#hp-main-menu-control').removeClass('selected');
      this.enableHeader();
   },

   // Called when ENTER keyboard control is pressed.
   onDefault: function() {
      if (this.isPopupOpen())
      {
         // Do not attempt to pass event since we interpret no event as ENTER being pressed.
         this.oVPopup.onDefault();
      }
   },
   
   // Called when ESC keyboard control is pressed.
   onDismissPopup: function() {
      if (this.isPopupOpen())
      {
         this.oVPopup.onDismiss();
      }
   },

   resizeWidth: function() {
      AVResize.prototype.resizeWidth.call(this);
      var nWidth = this.model.get('nWidth');
      if (IsDefined(this.oBV))
      {
         this.oBV.model.set('nWidth', nWidth);
         
         // Resize main menu control.
         var oMainMenuControl = this.$('#hp-main-menu-control');
         var oSecondary = this.$('.hp-header-secondary');
         var nSecondaryOuterWidth = oSecondary.outerWidth(true);
         var nBorderLeftWidth = parseInt(oMainMenuControl.css('border-left-width'));
         var nBorderRightWidth = parseInt(oMainMenuControl.css('border-right-width'));
         var nMainMenuControlWidth = nWidth - nSecondaryOuterWidth - nBorderLeftWidth - nBorderRightWidth;
         oMainMenuControl.width(nMainMenuControlWidth);
      }
      if (this.isPopupOpen())
      {
         this.oVPopup.model.set('nWidth', nWidth);
      }
   },
   
   resizeHeight: function() {
      AVResize.prototype.resizeHeight.call(this);
      var nHeight = this.model.get('nHeight');
      if (IsDefined(this.oBV))
      {
         var bannerHeight = this.$('#hp-banner-container').outerHeight(true);
         bannerHeight = ( bannerHeight == null ) ? 60 : bannerHeight; 
         var nBodyHeight = nHeight - bannerHeight;
         this.oBV.model.set('nHeight', nBodyHeight);
      }
      if (this.isPopupOpen())
      {
         this.oVPopup.model.set('nHeight', nHeight);
      }
   }
});